from __future__ import absolute_import

from django.conf.urls import patterns, url
from django.core.urlresolvers import reverse_lazy

from .views import AboutView

urlpatterns = patterns('common.views',
    url(r'^about/$', AboutView.as_view(), name='about'),
    url(r'^password/change/done/$', 'password_change_done', (), name='password_change_done'),
)

urlpatterns += patterns('',
    url(r'^login/$', 'django.contrib.auth.views.login', {'template_name': 'common/login.html'}, name='login_view'),
    url(r'^logout/$', 'django.contrib.auth.views.logout', {'next_page': reverse_lazy('home')}, name='logout_view'),

    url(r'^password/change/$', 'django.contrib.auth.views.password_change', {'template_name': 'common/password_change_form.html', 'post_change_redirect': reverse_lazy('password_change_done')}, name='password_change_view'),
    url(r'^password/reset/$', 'django.contrib.auth.views.password_reset', {'email_template_name': 'common/password_reset_email.html', 'template_name': 'common/password_reset_form.html', 'post_reset_redirect': '/password/reset/done'}, name='password_reset_view'),
    url(r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', 'django.contrib.auth.views.password_reset_confirm', {'template_name': 'common/password_reset_confirm.html', 'post_reset_redirect': '/password/reset/complete/'}, name='password_reset_confirm_view'),
    url(r'^password/reset/complete/$', 'django.contrib.auth.views.password_reset_complete', {'template_name': 'common/password_reset_complete.html'}, name='password_reset_complete_view'),
    url(r'^password/reset/done/$', 'django.contrib.auth.views.password_reset_done', {'template_name': 'common/password_reset_done.html'}, name='password_reset_done_view'),
)

urlpatterns += patterns('',
    url(r'^set_language/$', 'django.views.i18n.set_language', name='set_language'),
)



from django.utils.translation import ugettext_lazy as _

import assets
import inventory
import movements

from common.api import register_menu

from assets.models import Item, Person, ItemGroup, State
from inventory.models import ItemTemplate, Inventory, InventoryTransaction, Supplier, Location

register_menu([
    {'text': _(u'home'), 'view': 'home', 'famfam': 'house', 'position':0},

    {'text':_(u'setup'), 'view': 'location_list', 'links': [
        inventory.location_list, assets.state_list,
        movements.purchase_request_state_list,
        movements.purchase_order_state_list,
        movements.purchase_order_item_state_list,
    ],'famfam': 'cog', 'name': 'setup','position': 7},

    #{'text': _(u'tools'), 'view': 'import_wizard', 'links': [
    #    {'text': _(u'import'), 'view': 'import_wizard', 'famfam': 'lightning_add'},
    #],'famfam': 'wrench', 'name': 'tools','position': 6},

    {'text': _(u'about'), 'view': 'about', 'position': 8},
])



import csv

from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_unicode

def perform_import(csvfilename, model, mappings, dialect_settings=None, start_row=1, dryrun=True):
    csvfile = open(csvfilename, 'rb')
    dialect = csv.Sniffer().sniff(csvfile.read(1024))
    if dialect_settings:
        dialect.delimiter = str(dialect_settings['dialect_delimiter'])
        dialect.doublequote = dialect_settings['dialect_doublequote']
        dialect.escapechar = str(dialect_settings['dialect_escapechar'])
        dialect.quotechar = str(dialect_settings['dialect_quotechar'])
        dialect.skipinitialspace = dialect_settings['dialect_skipinitialspace']
        
    csvfile.seek(0)
    
    try:
        csvfd = file(csvfilename, 'r')
    except IOError:
        self.error(_(u'Could not open specified csv file, %s, or it does not exist') % datafile, 0)
    else:
        # CSV Reader returns an iterable, but as we possibly need to
        # perform list commands and since list is an acceptable iterable, 
        # we'll just transform it.
        csvfile = list(csv.reader(csvfd, dialect=dialect))

    model_fields = model._meta.init_name_map()
    imported_lines = 0
    errors = 0
    results = []
    for line, column in enumerate(csvfile[start_row-1:], start_row):
        column = [smart_unicode(c) for c in column]
        model_line = {}
        line_error = False
        for field_exp in mappings:
            if field_exp['enabled']:
                expression = None
                try:
                    field = model_fields[field_exp['model_field']][0]

                    expression = eval(field_exp['expression'], {'csv_column':column})
                    if hasattr(field, 'related'):
                        arguments = {field_exp['arguments']:expression}
                        value = field.related.parent_model.objects.get(**arguments)
                    else:
                        value = expression

                    model_line[field_exp['model_field']] = value

                except Exception, err:
                    value = None
                    line_error = True
                    errors += 1
                    results.append(_(u'Foreign key fetch error, line: %(line)s, expression: %(exp)s, error: %(err)s') % {'line':line, 'exp':expression, 'err':err})
        try:
            if not line_error:
                if dryrun:
                    entry = model(**model_line)
                else:
                    entry = model.objects.create(**model_line)

                imported_lines += 1
        except Exception, err:
            errors += 1
            results.append(_(u'Import error, line: %(line)s, error: %(err)s') % {'line':line, 'err':err})
        

    results.append(_(u'Processed %s lines.') % len(csvfile[start_row-1:]))
    results.append(_(u'Imported %s lines.') % imported_lines)
    results.append(_(u'There were %s errors.') % errors)
    
    csvfd.close()
    return results



from django.db import models
from django.contrib.auth.models import User, UserManager
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from dynamic_search.api import register
from inventory.models import ItemTemplate, Location
from photos.models import GenericPhoto


class State(models.Model):
    name = models.CharField(max_length=32, verbose_name=_(u'Name'))
    exclusive = models.BooleanField(default=False, verbose_name=_(u'Exclusive'))

    class Meta:
        verbose_name = _(u'Asset state')
        verbose_name_plural = _(u'Asset states')

    def __unicode__(self):
        return '%s (%s)' % (self.name, self.exclusive and _(u'Exclusive') or _(u'Inclusive'))

    @models.permalink
    def get_absolute_url(self):
        return ('state_list', [])


class ItemStateManager(models.Manager):
    def states_for_item(self, item):
        return self.filter(item=item)


class ItemState(models.Model):
    item = models.ForeignKey('Item', verbose_name=_(u'Item'))
    state = models.ForeignKey(State, verbose_name=_(u'State'))
    date = models.DateField(auto_now_add=True, verbose_name=_(u'Date'))

    objects = ItemStateManager()

    class Meta:
        verbose_name = _(u'Item state')
        verbose_name_plural = _(u'Item state')

    def __unicode__(self):
        return _(u'%(asset)s, %(state)s since %(date)s') % {'asset': self.item, 'state': self.state.name, 'date': self.date}

    @models.permalink
    def get_absolute_url(self):
        return ('state_update', [str(self.id)])


class Item(models.Model):
    item_template = models.ForeignKey(ItemTemplate, related_name='items', verbose_name=_(u'Item template'))
    property_number = models.CharField(max_length=48, verbose_name=_(u'Asset number'))
    notes = models.TextField(blank=True, null=True, verbose_name=_(u'Notes'))
    serial_number = models.CharField(blank=True, max_length=48, null=True, verbose_name=_(u'Serial number'))
    location = models.ForeignKey(Location, blank=True, null=True, verbose_name=_(u'Location'))
    active = models.BooleanField(default=True)

    class Meta:
        ordering = ['property_number']
        verbose_name = _(u'Asset')
        verbose_name_plural = _(u'Assets')

    @models.permalink
    def get_absolute_url(self):
        return ('item_view', [str(self.id)])

    def __unicode__(self):
        states = ', '.join([itemstate.state.name for itemstate in ItemState.objects.states_for_item(self)])
        return "#%s, '%s' %s" % (self.property_number, self.item_template.description, states and '(%s)' % states)

    def is_orphan(self):
        if self.person_set.all():
            return False
        else:
            return True

    def get_owners(self):
        try:
            return self.person_set.all()
        except:
            return None

    def get_nonowners(self):
        return Person.objects.all().exclude(id__in=self.person_set.values_list('id', flat=True))

    def add_owner(self, person):
        if self not in person.inventory.all():
            person.inventory.add(self)

    def remove_owner(self, person):
        person.inventory.remove(self)

    def states(self):
        return [State.objects.get(pk=id) for id in self.itemstate_set.all().values_list('state', flat=True)]


class ItemGroup(models.Model):
    name = models.CharField(verbose_name=_(u'Name'), max_length=32)
    items = models.ManyToManyField(Item, blank=True, null=True, verbose_name=_(u'Item'))

    class Meta:
        ordering = ['name']
        verbose_name = _(u'Item group')
        verbose_name_plural = _(u'Item groups')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('group_view', [str(self.id)])


class Person(models.Model):
    last_name = models.CharField(verbose_name=_(u'Last name'), max_length=32)
    second_last_name = models.CharField(verbose_name=_(u'Second last name'), max_length=32, blank=True, null=True)
    first_name = models.CharField(verbose_name=_(u'First name'), max_length=32)
    second_name = models.CharField(verbose_name=_(u'Second name or initial'), max_length=32, blank=True, null=True)
    location = models.ForeignKey(Location, blank=True, null=True, verbose_name=_(u'Location'))
    inventory = models.ManyToManyField(Item, blank=True, null=True, verbose_name=_(u'Assigned assets'))

    class Meta:
        ordering = ['last_name', 'second_last_name', 'first_name', 'second_name']
        verbose_name = _(u'Person')
        verbose_name_plural = _(u'People')

    @models.permalink
    def get_absolute_url(self):
        return ('person_view', [str(self.id)])

    def __unicode__(self):
        if self.second_last_name:
            second_last_name = ' %s' % self.second_last_name
        else:
            second_last_name = ''

        if self.second_name:
            second_name = ' %s' % self.second_name
        else:
            second_name = ''

        return '%s%s, %s%s' % (self.last_name, second_last_name and second_last_name, self.first_name, second_name)


register(ItemState, _(u'Asset states'), ['state__name'])
register(Item, _(u'Assets'), ['property_number', 'notes', 'serial_number', 'person__first_name', 'person__last_name', 'person__second_last_name', 'person__second_name'])
register(ItemGroup, _(u'Asset groups'), ['name'])
register(Person, _(u'People'), ['last_name', 'second_last_name', 'first_name', 'second_name', 'location__name'])



"""
Django settings for Django Inventory project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import sys

_file_path = os.path.abspath(os.path.dirname(__file__)).split('/')

BASE_DIR = '/'.join(_file_path[0:-2])

MEDIA_ROOT = os.path.join(BASE_DIR, 'django_inventory', 'media')

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '4#8p2r=zfa=t@3@@v5v*)i=kvnuyxae_yyvqkyv4!opcwad6@+'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = (
    # Django
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',
    # 3rd party
    'south',
    'pagination',
    'photologue',
    # Project
    'photos',
    'common',
    'generic_views',
    'inventory',
    'assets',
    'dynamic_search',
    #'importer',
    'movements',
    'main',
    'web_theme',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'common.middleware.login_required_middleware.LoginRequiredMiddleware',
    'pagination.middleware.PaginationMiddleware',
)

ROOT_URLCONF = 'django_inventory.urls'

WSGI_APPLICATION = 'django_inventory.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(MEDIA_ROOT, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Custom settings section

from django.core.urlresolvers import reverse_lazy

PROJECT_TITLE = 'Django Inventory'
PROJECT_NAME = 'django_inventory'

ugettext = lambda s: s

LANGUAGES = (
    ('es', ugettext('Spanish')),
    ('en', ugettext('English')),
    ('ru', ugettext('Russian')),
)

SITE_ID = 1

STATIC_URL = '/static/'

sys.path.append(os.path.join(BASE_DIR, 'django_inventory', 'apps'))

STATIC_ROOT = os.path.join(MEDIA_ROOT, 'static')

MEDIA_URL = '/site_media/'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    ('django.template.loaders.cached.Loader', (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )),
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.i18n',
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder',
)

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(BASE_DIR, 'django_inventory', 'templates')
)

LOGIN_URL = reverse_lazy('login_view')
LOGIN_REDIRECT_URL = reverse_lazy('home')

# --------- Django compressor -------------
COMPRESS_PARSER = 'compressor.parser.HtmlParser'
COMPRESS_CSS_FILTERS = ['compressor.filters.css_default.CssAbsoluteFilter', 'compressor.filters.cssmin.CSSMinFilter']
COMPRESS_ENABLED = False

# -------- LoginRequiredMiddleware ----------
LOGIN_EXEMPT_URLS = (
    r'^favicon\.ico$',
    r'^about\.html$',
    r'^legal/',  # allow the entire /legal/* subsection
    r'^static/',

    r'^accounts/register/$',
    r'^accounts/register/complete/$',
    r'^accounts/register/closed/$',

    r'^accounts/activate/complete/',
    r'^accounts/activate/(?P<activation_key>\w+)/$',

    r'^password/reset/$',
    r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
    r'^password/reset/complete/$',
    r'^password/reset/done/$',
)
# --------- Generic photos --------------
# PHOTOS_MAX_PHOTO_SIZE = 1000000
# INVENTORY_MAX_TEMPLATE_PHOTOS = 5
# ASSETS_MAX_ASSET_PHOTOS = 5
# ASSETS_MAX_PERSON_PHOTOS = 5
# --------- Pagination ------------------
PAGINATION_DEFAULT_PAGINATION = 10
# --------- Web theme app ---------------
WEB_THEME = 'warehouse'



from __future__ import absolute_import

from django import forms

from .models import GenericPhoto


class PhotoForm(forms.ModelForm):
    class Meta:
        model = GenericPhoto
        exclude = ('main', 'content_type', 'object_id', 'crop_from', 'effect')



from __future__ import absolute_import

from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse, reverse_lazy
from django.db.models import Q
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.utils.translation import ugettext as _

from photos.views import generic_photos
from generic_views.views import (GenericCreateView, GenericDetailView,
                                 GenericDeleteView, GenericListView,
                                 GenericUpdateView, generic_assign_remove)
from inventory import location_filter

from . import state_filter
from .forms import (ItemForm, ItemForm_view, ItemGroupForm, ItemGroupForm_view,
                    PersonForm, PersonForm_view)
from .models import Person, Item, ItemGroup, ItemState, State


class AssetCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Asset')}
    form_class = ItemForm


class AssetDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Asset')}
    model = Item
    success_url = reverse_lazy('item_list')


class AssetDetailView(GenericDetailView):
    extra_context = {
        'object_name': _(u'Asset'),
        'sidebar_subtemplates': ['photos/generic_photos_subtemplate.html', 'assets/state_subtemplate.html'],
        'extra_fields': [{'field': 'get_owners', 'label': _(u'Assigned to:')}]
    }
    form_class = ItemForm_view
    model = Item


class AssetOrphanListView(GenericListView):
    extra_context = {'title': _(u'Orphan assets')}
    list_filters = [location_filter]
    queryset = Item.objects.filter(person=None)


class AssetListView(GenericListView):
    extra_context = {'title': _(u'Assets')}
    list_filters = [location_filter, state_filter]
    model = Item


class AssetUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Asset')}
    form_class = ItemForm
    model = Item


class GroupCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Item group')}
    form_class = ItemGroupForm


class GroupListView(GenericListView):
    extra_context = {'title': _(u'Item groups')}
    model = ItemGroup


class GroupDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Item group')}
    model = ItemGroup
    success_url = reverse_lazy('group_list')


class GroupDetailView(GenericDetailView):
    extra_context = {'object_name': _(u'Item group')}
    form_class = ItemGroupForm_view
    model = ItemGroup


class PersonCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Person')}
    form_class = PersonForm


class PersonDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Person')}
    model = Person
    success_url = reverse_lazy('person_list')


class PersonDetailView(GenericDetailView):
    extra_context = {'sidebar_subtemplates': ['photos/generic_photos_subtemplate.html']}
    form_class = PersonForm_view
    model = Person


class PersonListView(GenericListView):
    extra_context = {'title': _(u'People')}
    list_filters = [location_filter]
    model = Person


class PersonUpdateView(GenericUpdateView):
    form_class = PersonForm
    model = Person


class StateCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Asset state')}
    model = State


class StateDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Asset state')}
    model = State
    success_url = reverse_lazy('state_list')


class StateListView(GenericListView):
    extra_context = {'title': _(u'Asset states')}
    model = State


class StateUpdateView(GenericUpdateView):
    model = State


def person_assign_remove_item(request, object_id):
    person = get_object_or_404(Person, pk=object_id)

    return generic_assign_remove(
        request,
        title=_(u'Assign assets to the person: <a href="%(url)s">%(obj)s</a>' % {'url': person.get_absolute_url(), 'obj': person}),
        obj=person,
        left_list_qryset=Item.objects.exclude(person=object_id),
        right_list_qryset=person.inventory.all(),
        add_method=person.inventory.add,
        remove_method=person.inventory.remove,
        left_list_title=_(u'Unassigned assets'),
        right_list_title=_(u'Assigned assets'),
        item_name=_(u'assets'),
        list_filter=[location_filter]
    )


def item_assign_remove_person(request, object_id):
    obj = get_object_or_404(Item, pk=object_id)

    return generic_assign_remove(
        request,
        title=_(u'Assign people to the asset: <a href="%(url)s">%(obj)s</a>' % {'url': obj.get_absolute_url(), 'obj': obj}),
        obj=obj,
        left_list_qryset=obj.get_nonowners(),
        right_list_qryset=obj.get_owners(),
        add_method=obj.add_owner,
        remove_method=obj.remove_owner,
        left_list_title=_(u'People that don\'t have this asset'),
        right_list_title=_(u'People that have this asset'),
        item_name=_(u'people'),
        list_filter=[location_filter])


def item_setstate(request, object_id, state_id):
    item = get_object_or_404(Item, pk=object_id)
    state = get_object_or_404(State, pk=state_id)

    if state.id in ItemState.objects.states_for_item(item).values_list('state', flat=True):
        messages.error(request, _(u'This asset has already been marked as "%s".') % state.name)
        return HttpResponseRedirect(reverse('item_view', args=[item.id]))

    next = reverse('item_view', args=[item.id])
    data = {
        'object': item,
        'title': _(u'Are you sure you wish to mark this asset as "%s"?') % state.name,
    }

    if state.exclusive:
        data['message'] = _(u'Any other states this asset may be marked as, will be cleared.')

    if request.method == 'POST':
        if state.exclusive:
            for item_state in ItemState.objects.states_for_item(item):
                item_state.delete()
        else:
            exclusive_state = ItemState.objects.states_for_item(item).filter(state__exclusive=True)
            if exclusive_state:
                messages.error(request, _(u'This asset has already been exclusively marked as "%s".  Clear this state first.') % exclusive_state[0].state.name)
                return HttpResponseRedirect(reverse('item_view', args=[item.id]))

        new = ItemState(item=item, state=state)
        new.save()

        messages.success(request, _(u'The asset has been marked as "%s".') % state.name)

        return HttpResponseRedirect(next)

    return render_to_response('generic_view/generic_confirm.html', data,
    context_instance=RequestContext(request))


def item_remove_state(request, object_id, state_id):
    item = get_object_or_404(Item, pk=object_id)
    state = get_object_or_404(State, pk=state_id)
    next = reverse('item_view', args=[item.id])

    item_state = ItemState.objects.filter(item=item, state=state)
    if not item_state:
        messages.error(request, _(u'This asset is not marked as "%s"') % state.name)
        return HttpResponseRedirect(next)

    data = {
        'object': item,
        'title': _(u'Are you sure you wish to unmark this asset as "%s"?') % state.name,
    }
    if request.method == 'POST':
        if item_state:
            try:
                item_state.delete()
                messages.success(request, _(u'The asset has been unmarked as "%s".') % state.name)
            except:
                messages.error(request, _(u'Unable to unmark this asset as "%s"') % state.name)

        return HttpResponseRedirect(next)

    return render_to_response('generic_view/generic_confirm.html', data,
    context_instance=RequestContext(request))


def group_assign_remove_item(request, object_id):
    obj = get_object_or_404(ItemGroup, pk=object_id)

    return generic_assign_remove(
        request,
        title=_(u'Assign assets to the group: <a href="%(url)s">%(obj)s</a>' % {'url': obj.get_absolute_url(), 'obj': obj}),
        obj=obj,
        left_list_qryset=Item.objects.exclude(itemgroup=obj),
        right_list_qryset=obj.items.all(),
        add_method=obj.items.add,
        remove_method=obj.items.remove,
        left_list_title=_(u'Unassigned assets'),
        right_list_title=_(u'Assigned assets'),
        item_name=_(u'Assets'),
        list_filter=[location_filter])



from django.template import Library, Node, Variable, VariableDoesNotExist
from django.utils.translation import ugettext as _

register = Library()


@register.inclusion_tag('generic_views/generic_form_subtemplate.html', takes_context=True)
def filter_form(context):
    new_context = context

    new_context.update({
        'form':context['filter_form'],
        'submit_method':'get',
        'title':_(u'Filter'),
    })
    return new_context




from __future__ import absolute_import

from django.contrib import admin

from .models import (PurchaseRequestStatus, PurchaseRequest,
                     PurchaseRequestItem, PurchaseOrderStatus,
                     PurchaseOrderItemStatus, PurchaseOrder, PurchaseOrderItem)


class PurchaseRequestItemInline(admin.StackedInline):
    model = PurchaseRequestItem
    extra = 1
    classes = ('collapse-open',)
    allow_add = True


class PurchaseRequestAdmin(admin.ModelAdmin):
    inlines = [PurchaseRequestItemInline]


class PurchaseOrderItemInline(admin.StackedInline):
    model = PurchaseOrderItem
    extra = 1
    classes = ('collapse-open',)
    allow_add = True


class PurchaseOrderAdmin(admin.ModelAdmin):
    inlines = [PurchaseOrderItemInline]


admin.site.register(PurchaseRequestStatus)
admin.site.register(PurchaseRequest, PurchaseRequestAdmin)
admin.site.register(PurchaseRequestItem)
admin.site.register(PurchaseOrderStatus)
admin.site.register(PurchaseOrderItemStatus)
admin.site.register(PurchaseOrder, PurchaseOrderAdmin)
admin.site.register(PurchaseOrderItem)



"""Common abstract classes for forms."""
try:
    import cPickle as pickle
except ImportError:
    import pickle

from django import forms
from django.conf import settings
from django.contrib.formtools.wizard import FormWizard
from django.forms.forms import BoundField
from django.forms.formsets import BaseFormSet
from django.utils.hashcompat import md5_constructor

__all__ = ('security_hash', 'BoundFormWizard')

def security_hash(request, form, exclude=None, *args):
    """Calculates a security hash for the given Form/FormSet instance.

    This creates a list of the form field names/values in a deterministic
    order, pickles the result with the SECRET_KEY setting, then takes an md5
    hash of that.
    """

    data = []
    if exclude is None:
        exclude = ()
    if isinstance(form, BaseFormSet):
        for _form in form.forms + [form.management_form]:
            for bf in _form:
                value = bf.field.clean(bf.data) or ''
                if isinstance(value, basestring):
                    value = value.strip()
                data.append((bf.name, value))
    else:
        for bf in form:
            if bf.name in exclude:
                continue
            value = bf.field.clean(bf.data) or ''
            if isinstance(value, basestring):
                value = value.strip()
            data.append((bf.name, value))
    data.extend(args)
    data.append(settings.SECRET_KEY)

    # Use HIGHEST_PROTOCOL because it's the most efficient. It requires
    # Python 2.3, but Django requires 2.3 anyway, so that's OK.
    pickled = pickle.dumps(data, pickle.HIGHEST_PROTOCOL)

    return md5_constructor(pickled).hexdigest()

class BoundFormWizard(FormWizard):
    """Render prev_fields as a list of bound form fields in the template
    context rather than raw html."""
    def security_hash(self, request, form):
        """Calculates the security hash for the given HttpRequest and
        Form/FormSet instances.

        Subclasses may want to take into account request-specific information,
        such as the IP address.
        """
        return security_hash(request, form)

    def render(self, form, request, step, context=None):
        "Renders the given Form object, returning an HttpResponse."
        old_data = request.POST
        prev_fields = []
        if old_data:
            for i in range(step):
                old_form = self.get_form(i, old_data)
                hash_name = 'hash_%s' % i
                if isinstance(old_form, BaseFormSet):
                    for _form in old_form.forms + [old_form.management_form]:
                        prev_fields.extend([bf for bf in _form])
                else:
                    prev_fields.extend([bf for bf in old_form])
                hash_field = forms.Field(initial=old_data.get(hash_name,
                    self.security_hash(request, old_form)))
                bf = BoundField(forms.Form(), hash_field, hash_name)
                prev_fields.append(bf)
        return self.render_template(request, form, prev_fields, step, context)

# vim: ai ts=4 sts=4 et sw=4



from __future__ import absolute_import

from django import forms
from django.utils.translation import ugettext_lazy as _

from generic_views.forms import DetailForm

from .models import (Inventory, InventoryTransaction, ItemTemplate, Location,
                     Log, Supplier)


class LocationForm_view(DetailForm):
    class Meta:
        model = Location


class ItemTemplateForm(forms.ModelForm):
    class Meta:
        model = ItemTemplate
        exclude = ('photos', 'supplies', 'suppliers')


class ItemTemplateForm_view(DetailForm):
    class Meta:
        model = ItemTemplate
        exclude = ('photos',)


class LogForm(forms.ModelForm):
    class Meta:
        model = Log


class InventoryForm(forms.ModelForm):
    class Meta:
        model = Inventory


class InventoryForm_view(DetailForm):
    class Meta:
        model = Inventory


class InventoryTransactionForm(forms.ModelForm):
    class Meta:
        model = InventoryTransaction


class SupplierForm(forms.ModelForm):
    class Meta:
        model = Supplier



from __future__ import absolute_import

from django.template import Library, Node, Variable
from django.template.defaultfilters import stringfilter

from ..models import GenericPhoto

register = Library()


@register.filter
def get_photos_for_object(value):
    return GenericPhoto.objects.photos_for_object(value)



def encapsulate(function):
    # Workaround Django ticket 15791
    # Changeset 16045
    # http://stackoverflow.com/questions/6861601/cannot-resolve-callable-context-variable/6955045#6955045
    return lambda: function



from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _

from common.api import register_links, register_menu

from .models import Location, ItemTemplate, Inventory, InventoryTransaction, Supplier

inventory_list = {'text': _('View all inventories'), 'view': 'inventory_list', 'famfam': 'package_go'}
inventory_create = {'text': _('Create new inventory'), 'view': 'inventory_create', 'famfam': 'package_add'}
inventory_update = {'text': _(u'Edit'), 'view': 'inventory_update', 'args': 'object.id', 'famfam': 'package_green'}
inventory_delete = {'text': _(u'Delete'), 'view': 'inventory_delete', 'args': 'object.id', 'famfam': 'package_delete'}
inventory_create_transaction = {'text': _('Add transaction'), 'view': 'inventory_create_transaction', 'args': 'object.id', 'famfam': 'book_add'}
inventory_view = {'text': _(u'Details'), 'view': 'inventory_view', 'args': 'object.id', 'famfam': 'package_go'}
inventory_list_transactions = {'text': _(u'Inventory transactions'), 'view': 'inventory_list_transactions', 'args': 'object.id', 'famfam': 'book_go'}

inventory_transaction_update = {'text': _(u'Edit'), 'view': 'inventory_transaction_update', 'args': 'object.id', 'famfam': 'book_add'}
inventory_transaction_delete = {'text': _(u'Delete'), 'view': 'inventory_transaction_delete', 'args': 'object.id', 'famfam': 'book_delete'}
inventory_transaction_view = {'text': _(u'Details'), 'view': 'inventory_transaction_view', 'args': 'object.id', 'famfam': 'book_go'}

location_list = {'text': _('Locations'), 'view': 'location_list', 'famfam': 'map'}
location_create = {'text': _(u'Create new location'), 'view': 'location_create', 'famfam': 'map_add'}
location_update = {'text': _(u'Edit'), 'view': 'location_update', 'args': 'object.id', 'famfam': 'map_edit'}
location_delete = {'text': _(u'Delete'), 'view': 'location_delete', 'args': 'object.id', 'famfam': 'map_delete'}

supplier_create = {'text': _('Create new supplier'), 'view': 'supplier_create', 'famfam': 'lorry_add'}
supplier_list = {'text': _('Suppliers'), 'view': 'supplier_list', 'famfam': 'lorry'}
supplier_update = {'text': _('Edit'), 'view': 'supplier_update', 'args': 'object.id', 'famfam': 'lorry'}
supplier_delete = {'text': _('Delete'), 'view': 'supplier_delete', 'args': 'object.id', 'famfam': 'lorry_delete'}
supplier_assign_itemtemplate = {'text': _(u'Assign templates'), 'view': 'supplier_assign_itemtemplates', 'args': 'object.id', 'famfam': 'page_go'}
supplier_purchase_orders = {'text': _(u'Related purchase orders'), 'view': 'supplier_purchase_orders', 'args': 'object.id', 'famfam': 'cart_go'}

template_list = {'text': _('View all'), 'view': 'template_list', 'famfam': 'page_go'}
template_create = {'text': _('Create new template'), 'view': 'template_create', 'famfam': 'page_add'}
template_orphan_list = {'text': _('Orphans templates'), 'view': 'template_orphans_list'}
template_update = {'text': _(u'Edit'), 'view': 'template_update', 'args': 'object.id', 'famfam': 'page_edit'}
template_delete = {'text': _(u'Delete'), 'view': 'template_delete', 'args': 'object.id', 'famfam': 'page_delete'}
template_photos = {'text': _(u'Add / remove photos'), 'view': 'template_photos', 'args': 'object.id', 'famfam': 'picture_go'}
template_assets = {'text': _(u'Related assets'), 'view': 'template_items_list', 'args': 'object.id', 'famfam': 'computer_go'}
template_assign_supplies = {'text': _(u'Assign supplies'), 'view': 'template_assign_supply', 'args': 'object.id', 'famfam': 'monitor'}
template_assign_suppliers = {'text': _(u'Assign suppliers'), 'view': 'template_assign_suppliers', 'args': 'object.id', 'famfam': 'lorry_go'}

jump_to_template = {'text': _(u'Template'), 'view': 'template_view', 'args': 'object.supply.id', 'famfam': 'page_go'}
jump_to_inventory = {'text': _(u'Return to inventory'), 'view': 'inventory_view', 'args': 'object.inventory.id', 'famfam': 'package_go'}

template_menu_links = [template_list, template_orphan_list, supplier_list]
inventory_menu_links = [
    inventory_list,
]

location_filter = {'name': 'Location', 'title': _(u'location'), 'queryset': Location.objects.all(), 'destination': 'location'}

register_links(['template_list', 'template_create', 'template_view', 'template_orphans_list', 'template_update', 'template_delete', 'template_photos', 'template_assign_supply', 'template_assign_suppliers'], [template_create], menu_name='sidebar')
register_links(ItemTemplate, [template_update, template_delete, template_photos, template_assets, template_assign_supplies, template_assign_suppliers])

register_links(['supplier_list', 'supplier_create', 'supplier_update', 'supplier_view', 'supplier_delete', 'supplier_assign_itemtemplates'], [supplier_create], menu_name='sidebar')
register_links(Supplier, [supplier_update, supplier_delete, supplier_assign_itemtemplate, supplier_purchase_orders])

register_links(['inventory_view', 'inventory_list', 'inventory_create', 'inventory_update', 'inventory_delete'], [inventory_create], menu_name='sidebar')
register_links(Inventory, [inventory_update, inventory_delete, inventory_list_transactions, inventory_create_transaction])
register_links(Inventory, [inventory_view], menu_name='sidebar')

register_links(['inventory_transaction_update', 'inventory_transaction_delete', 'inventory_transaction_view'], [inventory_create_transaction], menu_name='sidebar')
register_links(InventoryTransaction, [inventory_transaction_view, inventory_transaction_update, inventory_transaction_delete, jump_to_template])
register_links(InventoryTransaction, [jump_to_inventory], menu_name='sidebar')

register_links(['location_list', 'location_create', 'location_update', 'location_delete'], [location_create], menu_name='sidebar')
register_links(Location, [location_update, location_delete])

register_menu([
    {'text': _('Templates'), 'view': 'template_list', 'links': template_menu_links, 'famfam': 'page', 'position': 1},
    {'text': _('Inventories'), 'view': 'inventory_list', 'links': inventory_menu_links, 'famfam': 'package', 'position': 4},
])



from django.contrib import admin

from .models import State, ItemState, Item, ItemGroup, Person

admin.site.register(State)
admin.site.register(ItemState)
admin.site.register(Item)
admin.site.register(ItemGroup)
admin.site.register(Person)



from __future__ import absolute_import

from django.core.urlresolvers import reverse, reverse_lazy
from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, ListView

from assets.models import Person, Item, ItemGroup
from generic_views.views import (GenericCreateView, GenericDeleteView,
                                 GenericDetailView, GenericListView,
                                 GenericUpdateView, generic_assign_remove)
from inventory import location_filter
from photos.views import generic_photos

from .forms import (InventoryForm, InventoryForm_view, InventoryTransactionForm,
                    ItemTemplateForm, ItemTemplateForm_view, LocationForm_view,
                    SupplierForm)
from .models import (Inventory, InventoryTransaction, ItemTemplate, Location,
                     Supplier)


class InventoryCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Inventory')}
    model = Inventory


class InventoryDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Inventory')}
    model = Inventory
    success_url = reverse_lazy('inventory_list')


class InventoryDetailView(GenericDetailView):
    extra_context = {
        'object_name': _(u'Inventory'),
    }
    form_class = InventoryForm_view
    model = Inventory

    def get_context_data(self, **kwargs):
        context = super(InventoryDetailView, self).get_context_data(**kwargs)
        supply_qty = {}
        for transaction in self.get_object().transactions.all():
            if transaction.supply in supply_qty:
                supply_qty[transaction.supply] = supply_qty[transaction.supply] + transaction.quantity
            else:
                supply_qty[transaction.supply] = transaction.quantity

        supplies_list = [{'item_template': item_template, 'qty': qty} for item_template, qty in supply_qty.items()]

        context['title'] = _(u'current balances for inventory: %s') % self.get_object()
        context['subtemplates_dict'] = [
            {
                'title': _(u'Items'),
                'name': 'generic_views/generic_list_subtemplate.html',
                'object_list': supplies_list,
                'main_object': 'item_template',
                'extra_columns': [{'name': _(u'quantity'), 'attribute': 'qty'}],
            }
        ]
        return context


class InventoryListView(GenericListView):
    extra_context = {
        'title': _(u'Inventories'),
        'extra_columns': [{'name': _(u'Location'), 'attribute': 'location'}]
    }
    model = Inventory
    template_name = 'generic_views/generic_list.html'


class InventoryUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Inventory')}
    form_class = InventoryForm
    model = Inventory


class LocationCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Location')}
    model = Location


class LocationDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Location')}
    model = Location
    success_url = reverse_lazy('location_list')


class LocationDetailView(GenericDetailView):
    form_class = LocationForm_view
    model = Location


class LocationListView(GenericListView):
    extra_context = {'title': _(u'Locations')}
    model = Location


class LocationUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Location')}
    model = Location


class SupplierCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Supplier')}
    form_class = SupplierForm


class SupplierDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Supplier')}
    model = Supplier
    success_url = reverse_lazy('supplier_list')


class SupplierDetailView(GenericDetailView):
    form_class = SupplierForm
    model = Supplier


class SupplierListView(GenericListView):
    extra_context = {
        'title': _(u'Suppliers')
    }
    model = Supplier


class SupplierPurchaseOrdersListView(GenericListView):
    def get_supplier(self):
        return get_object_or_404(Supplier, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_supplier().purchase_orders.all()

    def get_context_data(self, **kwargs):
        context = super(SupplierPurchaseOrdersListView, self).get_context_data(**kwargs)
        context['title'] = _(u'Purchase orders from supplier: %s') % self.get_supplier()
        return context


class SupplierUpdateView(GenericUpdateView):
    form_class = SupplierForm
    model = Supplier


class TemplateCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Item template')}
    form_class = ItemTemplateForm


class TemplateDeleteView(GenericDeleteView):
    extra_context = {
        'object_name': _(u'Item template'),
        'message': _(u'Will be deleted from any user that may have it assigned and from any item group.'),
    }
    model = ItemTemplate
    success_url = reverse_lazy('template_list')


class TemplateItemsListView(GenericListView):
    def get_template(self):
        return get_object_or_404(ItemTemplate, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_template().items.all()

    def get_context_data(self, **kwargs):
        context = super(TemplateItemsListView, self).get_context_data(**kwargs)
        context['title'] = _(u'Assets that use the template: %s') % self.get_template()
        return context


class TemplateListView(GenericListView):
    extra_context = {'title': _(u'Item template')}
    model = ItemTemplate


class TemplateOrphanListView(GenericListView):
    extra_context = {'title': _('Orphan templates')}
    queryset = ItemTemplate.objects.filter(items=None)


class TemplateUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Item template')}
    form_class = ItemTemplateForm
    model = ItemTemplate


class TemplateDetailView(GenericDetailView):
    form_class = ItemTemplateForm_view
    extra_context = {
        'object_name': _(u'Item template'),
        'sidebar_subtemplates': ['photos/generic_photos_subtemplate.html']
    }
    model = ItemTemplate


class TransactionDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Transaction')}
    model = InventoryTransaction
    success_url = reverse_lazy('inventory_list')


class TransactionDetailView(GenericDetailView):
    extra_context = {'object_name': _(u'Transaction')}
    form_class = InventoryTransactionForm
    model = InventoryTransaction


class TransactionUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Transaction')}
    model = InventoryTransaction


def supplier_assign_remove_itemtemplates(request, object_id):
    obj = get_object_or_404(Supplier, pk=object_id)

    return generic_assign_remove(
        request,
        title=_(u'Assign templates to the supplier: <a href="%(url)s">%(obj)s</a>' % {'url': obj.get_absolute_url(), 'obj': obj}),
        obj=obj,
        left_list_qryset=ItemTemplate.objects.exclude(suppliers=obj),
        right_list_qryset=obj.itemtemplate_set.all(),
        add_method=obj.itemtemplate_set.add,
        remove_method=obj.itemtemplate_set.remove,
        left_list_title=_(u'Unassigned templates'),
        right_list_title=_(u'Assigned templates'),
        item_name=_(u"templates"),
    )


def template_assign_remove_supply(request, object_id):
    obj = get_object_or_404(ItemTemplate, pk=object_id)

    return generic_assign_remove(
        request,
        title=_(u'Assign supplies to the template: <a href="%(url)s">%(obj)s</a>' % {'url': obj.get_absolute_url(), 'obj': obj}),
        obj=obj,
        left_list_qryset=ItemTemplate.objects.exclude(supplies=obj).exclude(pk=obj.pk),
        right_list_qryset=obj.supplies.all(),
        add_method=obj.supplies.add,
        remove_method=obj.supplies.remove,
        left_list_title=_(u'Unassigned supplies'),
        right_list_title=_(u'Assigned supplies'),
        item_name=_(u"supplies"))


def template_assign_remove_suppliers(request, object_id):
    obj = get_object_or_404(ItemTemplate, pk=object_id)

    return generic_assign_remove(
        request,
        title=_(u'Assign suppliers to the template: <a href="%(url)s">%(obj)s</a>' % {'url': obj.get_absolute_url(), 'obj': obj}),
        obj=obj,
        left_list_qryset=Supplier.objects.exclude(itemtemplate=obj),
        right_list_qryset=obj.suppliers.all(),
        add_method=obj.suppliers.add,
        remove_method=obj.suppliers.remove,
        left_list_title=_(u'Unassigned suppliers'),
        right_list_title=_(u'Assigned suppliers'),
        item_name=_(u"suppliers"))


def inventory_list_transactions(request, object_id):
    inventory = get_object_or_404(Inventory, pk=object_id)
    form = InventoryForm_view(instance=inventory)

    return render_to_response('generic_views/generic_detail.html', {
        'object_name': _(u'Inventory'),
        'object': inventory,
        'form': form,
        'subtemplates_dict': [
            {
                'name': 'generic_views/generic_list_subtemplate.html',
                'title': _(u'Inventory transactions'),
                'object_list': inventory.transactions.all(),
                'hide_object': True,
                'extra_columns': [
                    {'name': _(u'Date'), 'attribute': 'date'},
                    {'name': _(u'Item'), 'attribute': 'supply'},
                    {'name': _(u'Qty'), 'attribute': 'quantity'},
                ],
            }
        ]
    }, context_instance=RequestContext(request))


def inventory_create_transaction(request, object_id):
    inventory = get_object_or_404(Inventory, pk=object_id)

    if request.method == 'POST':
        form = InventoryTransactionForm(request.POST)
        if form.is_valid():
            form.save()
            msg = _(u'The inventory transaction was created successfully.')
            messages.success(request, msg, fail_silently=True)
            return redirect('inventory_list_transactions', inventory.id)
    else:
        form = InventoryTransactionForm(initial={'inventory': inventory})

    return render_to_response('generic_views/generic_form.html', {
        'form': form,
        'object': inventory,
        'title': _(u'Add new transaction'),
    }, context_instance=RequestContext(request))



from django.contrib import messages
from django.shortcuts import redirect
from django.utils.translation import ugettext as _

from django.views.generic import TemplateView


class AboutView(TemplateView):
    template_name = 'main/about.html'


def password_change_done(request):
    messages.success(request, _(u'Your password has been successfully changed.'))
    return redirect('home')



from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _

from common.api import register_links

from .models import GenericPhoto

generic_photo_delete = {'text': _('Delete photo'), 'view': 'generic_photo_delete', 'args': 'object.id', 'famfam': 'picture_delete'}
generic_photo_mark_main = {'text': _('Mark photo as main'), 'view': 'generic_photo_mark_main', 'args': 'object.id', 'famfam': 'picture_edit'}

register_links(GenericPhoto, [generic_photo_mark_main, generic_photo_delete])
register_links(['generic_photo_mark_main', 'generic_photo_delete'], [generic_photo_mark_main, generic_photo_delete], menu_name='sidebar')



from __future__ import absolute_import

from .base import *

INTERNAL_IPS = ('127.0.0.1',)

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

try:
    import django_extensions
    INSTALLED_APPS += ('django_extensions',)
except ImportError:
    print 'django_extensions is not installed'

TEMPLATE_CONTEXT_PROCESSORS += ('django.core.context_processors.debug',)

WSGI_AUTO_RELOAD = True
if 'debug_toolbar' in INSTALLED_APPS:
    MIDDLEWARE_CLASSES += ('debug_toolbar.middleware.DebugToolbarMiddleware',)
    DEBUG_TOOLBAR_CONFIG = {
        'INTERCEPT_REDIRECTS': False,
    }



from __future__ import absolute_import

from django import forms
from django.utils.translation import ugettext_lazy as _

from generic_views.forms import DetailForm
from inventory.models import Inventory

from .models import (PurchaseRequest, PurchaseRequestItem, PurchaseOrder,
                     PurchaseOrderItem)


class PurchaseRequestForm(forms.ModelForm):
    class Meta:
        model = PurchaseRequest
        exclude = ('active',)


class PurchaseRequestForm_view(DetailForm):
    class Meta:
        model = PurchaseRequest
        exclude = ('active',)


class PurchaseRequestItemForm(forms.ModelForm):
    class Meta:
        model = PurchaseRequestItem


class PurchaseOrderForm(forms.ModelForm):
    class Meta:
        model = PurchaseOrder
        exclude = ('active',)


class PurchaseOrderForm_view(DetailForm):
    class Meta:
        model = PurchaseOrder
        exclude = ('active',)


class PurchaseOrderItemForm(forms.ModelForm):
    class Meta:
        model = PurchaseOrderItem
        exclude = ('active',)


class PurchaseOrderWizardItemForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(PurchaseOrderWizardItemForm, self).__init__(*args, **kwargs)
        if 'item' in self.initial:
            self.fields['template_id'].initial = self.initial['item'].item_template.id
            self.fields['name'].initial = self.initial['item'].item_template
            self.fields['supplier'].choices = self.initial['item'].item_template.suppliers.all().values_list('id', 'name')
            self.fields['qty'].initial = self.initial['item'].qty

    template_id = forms.CharField(widget=forms.HiddenInput)
    name = forms.CharField(label=_(u'Name'), required=False, widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    supplier = forms.ChoiceField(label=_(u'Suppliers'))
    qty = forms.CharField(label=_(u'Qty'))


class PurchaseOrderItemTransferForm(forms.Form):
    purchase_order_item_id = forms.CharField(widget=forms.HiddenInput)
    purchase_order_item = forms.CharField(label=_(u'Purchase order item'), widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    inventory = forms.ModelChoiceField(queryset=Inventory.objects.all(), help_text=_(u'Inventory that will receive the item.'))
    qty = forms.CharField(label=_(u'Qty received'))



from django.conf import settings

#Theme options are:
#amro, bec, bec-green, blue, default, djime-cerulean, drastic-dark,
#kathleene, olive, orange, red, reidb-greenish, warehouse

THEME = getattr(settings, 'WEB_THEME', 'default')



from __future__ import absolute_import

from django.conf.urls import patterns, url

from django.utils.translation import ugettext_lazy as _

from generic_views.views import generic_assign_remove

from . import purchase_order_state_filter, purchase_request_state_filter
from .forms import PurchaseOrderForm, PurchaseOrderItemForm, PurchaseRequestForm
from .models import (PurchaseOrder, PurchaseOrderItem, PurchaseOrderItemStatus,
                     PurchaseOrderStatus, PurchaseRequest, PurchaseRequestItem,
                     PurchaseRequestStatus)
from .views import (PurchaseRequestCreateView, PurchaseRequestDeleteView,
                    PurchaseRequestListView, PurchaseRequestUpdateView,
                    PurchaseRequestStateCreateView,
                    PurchaseRequestStateDeleteView,
                    PurchaseRequestStateUpdateView,
                    PurchaseRequestStateListView,
                    PurchaseRequestStateUpdateView,
                    PurchaseRequestItemDeleteView,
                    PurchaseRequestItemUpdateView,
                    PurchaseOrderCreateView, PurchaseOrderDeleteView,
                    PurchaseOrderListView, PurchaseOrderUpdateView,
                    PurchaseOrderItemDeleteView, PurchaseOrderItemUpdateView,
                    PurchaseOrderItemStatusCreateView,
                    PurchaseOrderItemStatusDeleteView,
                    PurchaseOrderItemStatusListView,
                    PurchaseOrderItemStatusUpdateView,
                    PurchaseOrderStatusCreateView,
                    PurchaseOrderStatusDeleteView, PurchaseOrderStatusListView,
                    PurchaseOrderStatusUpdateView)

urlpatterns = patterns('movements.views',
    url(r'^purchase/request/state/list/$', PurchaseRequestStateListView.as_view(), name='purchase_request_state_list'),
    url(r'^purchase/request/state/create/$', PurchaseRequestStateCreateView.as_view(), name='purchase_request_state_create'),
    url(r'^purchase/request/state/(?P<pk>\d+)/update/$', PurchaseRequestStateUpdateView.as_view(), name='purchase_request_state_update'),
    url(r'^purchase/request/state/(?P<pk>\d+)/delete/$', PurchaseRequestStateDeleteView.as_view(), name='purchase_request_state_delete'),

    url(r'^purchase/request/list/$', PurchaseRequestListView.as_view(), name='purchase_request_list'),
    url(r'^purchase/request/(?P<object_id>\d+)/$', 'purchase_request_view', (), 'purchase_request_view'),
    url(r'^purchase/request/create/$', PurchaseRequestCreateView.as_view(), name='purchase_request_create'),
    url(r'^purchase/request/(?P<pk>\d+)/update/$', PurchaseRequestUpdateView.as_view(), name='purchase_request_update'),
    url(r'^purchase/request/(?P<pk>\d+)/delete/$', PurchaseRequestDeleteView.as_view(), name='purchase_request_delete'),
    url(r'^purchase/request/(?P<object_id>\d+)/close/$', 'purchase_request_close', (), 'purchase_request_close'),
    url(r'^purchase/request/(?P<object_id>\d+)/open/$', 'purchase_request_open', (), 'purchase_request_open'),
    url(r'^purchase/request/(?P<object_id>\d+)/purchase_order_wizard/$', 'purchase_order_wizard', (), 'purchase_order_wizard'),

    url(r'^purchase/request/(?P<object_id>\d+)/add_item/$', 'purchase_request_item_create', (), 'purchase_request_item_create'),
    url(r'^purchase/request/item/(?P<pk>\d+)/update/$', PurchaseRequestItemUpdateView.as_view(), name='purchase_request_item_update'),
    url(r'^purchase/request/item/(?P<pk>\d+)/delete/$', PurchaseRequestItemDeleteView.as_view(), name='purchase_request_item_delete'),

    url(r'^purchase/order/status/list/$', PurchaseOrderStatusListView.as_view(), name='purchase_order_state_list'),
    url(r'^purchase/order/status/create/$', PurchaseOrderStatusCreateView.as_view(), name='purchase_order_state_create'),
    url(r'^purchase/order/status/(?P<pk>\d+)/update/$', PurchaseOrderStatusUpdateView.as_view(), name='purchase_order_state_update'),
    url(r'^purchase/order/status/(?P<pk>\d+)/delete/$', PurchaseOrderStatusDeleteView.as_view(), name='purchase_order_state_delete'),

    url(r'^purchase/order/list/$', PurchaseOrderListView.as_view(), name='purchase_order_list'),
    url(r'^purchase/order/(?P<object_id>\d+)/$', 'purchase_order_view', (), 'purchase_order_view'),
    url(r'^purchase/order/create/$', PurchaseOrderCreateView.as_view(), name='purchase_order_create'),
    url(r'^purchase/order/(?P<pk>\d+)/update/$', PurchaseOrderUpdateView.as_view(), name='purchase_order_update'),
    url(r'^purchase/order/(?P<pk>\d+)/delete/$', PurchaseOrderDeleteView.as_view(), name='purchase_order_delete'),
    url(r'^purchase/order/(?P<object_id>\d+)/close/$', 'purchase_order_close', (), 'purchase_order_close'),
    url(r'^purchase/order/(?P<object_id>\d+)/open/$', 'purchase_order_open', (), 'purchase_order_open'),
    url(r'^purchase/order/(?P<object_id>\d+)/add_item/$', 'purchase_order_item_create', (), 'purchase_order_item_create'),
    url(r'^purchase/order/(?P<object_id>\d+)/transfer/$', 'purchase_order_transfer', (), 'purchase_order_transfer'),

    url(r'^purchase/order/item/status/list/$', PurchaseOrderItemStatusListView.as_view(), name='purchase_order_item_state_list'),
    url(r'^purchase/order/item/status/create/$', PurchaseOrderItemStatusCreateView.as_view(), name='purchase_order_item_state_create'),
    url(r'^purchase/order/item/status/(?P<pk>\d+)/update/$', PurchaseOrderItemStatusUpdateView.as_view(), name='purchase_order_item_state_update'),
    url(r'^purchase/order/item/status/(?P<pk>\d+)/delete/$', PurchaseOrderItemStatusDeleteView.as_view(), name='purchase_order_item_state_delete'),

    url(r'^purchase/order/item/(?P<pk>\d+)/update/$', PurchaseOrderItemUpdateView.as_view(), name='purchase_order_item_update'),
    url(r'^purchase/order/item/(?P<pk>\d+)/delete/$', PurchaseOrderItemDeleteView.as_view(), name='purchase_order_item_delete'),
    url(r'^purchase/order/item/(?P<object_id>\d+)/close/$', 'purchase_order_item_close', (), 'purchase_order_item_close'),
    url(r'^purchase/order/item/(?P<object_id>\d+)/transfer/$', 'purchase_order_item_transfer', (), 'purchase_order_item_transfer'),
)



from __future__ import absolute_import

from django import forms
from django.utils.translation import ugettext_lazy as _

from generic_views.forms import DetailForm

from .models import Item, Person, ItemGroup


class ItemForm(forms.ModelForm):
    class Meta:
        model = Item
        exclude = ('photos', 'active')


class ItemForm_view(DetailForm):
    class Meta:
        model = Item
        exclude = ('photos', 'active')


class PersonForm(forms.ModelForm):
    class Meta:
        model = Person
        exclude = ('photos', 'inventory')


class PersonForm_view(DetailForm):
    class Meta:
        model = Person
        exclude = ('photos',)


class ItemGroupForm(forms.ModelForm):
    class Meta:
        model = ItemGroup
        exclude = ('items',)


class ItemGroupForm_view(DetailForm):
    class Meta:
        model = ItemGroup



import types

from django.core.urlresolvers import reverse
from django.conf import settings
from django.template.defaultfilters import stringfilter
from django.template import Library, Node, Variable, VariableDoesNotExist


register = Library()

class DynUrlNode(Node):
    def __init__(self, *args):
        self.name_var = Variable(args[0])
        if len(args)>1:
            #Process view arguments
            self.args = [Variable(a) for a in args[1].split(',')]
        else:
            self.args = []

    def render(self, context):
        name = self.name_var.resolve(context)
        args = [a.resolve(context) for a in self.args]
        try:
            return reverse(name, args = args)
        except:
            #Argument might be pointing to a context variable
            args = [Variable(a).resolve(context) for a in args]
            return reverse(name, args = args)


@register.tag
def dynurl(parser, token):
    args = token.split_contents()
    return DynUrlNode(*args[1:])


@register.filter
@stringfilter
def is_class(value, arg):
    return value[21:(value[21:].find(' ')+21)] == arg


def return_attrib(obj, attrib, arguments={}):
    try:
        if isinstance(obj, types.DictType) or isinstance(obj, types.DictionaryType):
            return obj[attrib]
        elif isinstance(attrib, types.FunctionType):
            return attrib(obj)
        else:
            result = reduce(getattr, attrib.split("."), obj)
            if isinstance(result, types.MethodType):
                if arguments:
                    return result(**arguments)
                else:
                    return result()
            else:
                return result
    except Exception, err:
        if settings.DEBUG:
            return "Error: %s; %s" % (attrib, err)
        else:
            pass

@register.filter
def object_property(value, arg):
    return return_attrib(value, arg)



import os
import csv

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.utils.simplejson import dumps, loads
from django.http import HttpResponseRedirect
from django.http import Http404
from django.contrib.formtools.wizard import FormWizard
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404, redirect

from api import perform_import
from wizard import BoundFormWizard

#TODO: Allow row 0 to be used as column names
#TODO: Load mapping in step 0 or reuse last import settings
#Ticket #7439 (new) FormWizard don't process multipart forms
#Ticket #11112 (new) Formsets not supported as steps in FormWizard   - DONE

def reduce_dict(d, l):
    return dict([(i, d[i]) for i in l])


class DocumentValidationError(forms.ValidationError):
    def __init__(self):
        msg = _(u'Only CSV files are valid uploads.')
        super(DocumentValidationError, self).__init__(msg)


class DocumentField(forms.FileField):
    """A validating PDF document upload field"""

    def clean(self, data, initial=None):
        f = super(DocumentField, self).clean(data, initial)
        ext = os.path.splitext(f.name)[1][1:].lower()

        if ext == 'csv' and f.content_type == 'text/csv':
            return f
        raise DocumentValidationError()


class DocumentForm(forms.Form):
    def __init__(self, *args, **kwargs):
        models = kwargs.pop('models', [])
        if models:
            choices = models
        else:
            qs=ContentType.objects.all()
            choices = sorted(["%s.%s" % (model.app_label, model.model) for model in qs])

        exclude = kwargs.pop('exclude', [])
        for exclusion in exclude:
            choices.remove(exclusion)

        super(DocumentForm, self).__init__(*args, **kwargs)
        
        capfirst = lambda x: x[0].upper() + x[1:]
        
        get_verbose_name = lambda x: getattr(x._meta, 'verbose_name', x) if hasattr(x, '_meta') else x
        
        names = [capfirst(get_verbose_name(ContentType.objects.get(app_label=model.split('.')[0], model=model.split('.')[1]).model_class())) for model in choices]
                
        self.fields['model_name'].choices=sorted(zip(choices, names), lambda x,y: 1 if x[1]>y[1] else -1)
        
    local_document = DocumentField(label=_(u'Local document'))
    model_name = forms.ChoiceField(label=_(u'Model'), help_text=_(u'Model that will receive the data.'))
    #import_mappings = forms.FileField(label=_(u'Import mappings file (optional)'), required=False)



class PreviewForm(forms.Form):
    preview_area = forms.CharField(label=_(u'Preview'), required=False, widget=forms.widgets.Textarea(attrs={'cols':80, 'rows':10}))
    start_row = forms.IntegerField(label=_(u'Start row'), initial=1)
    dialect_delimiter = forms.CharField(label=_('Delimiter'), max_length=1, help_text=_(u'A one-character string used to separate fields. It defaults to ",".'))
    dialect_quotechar = forms.CharField(label=_('Quote character'), max_length=1, help_text=_(u'A one-character string used to quote fields containing special characters, such as the delimiter or quote character, or which contain new-line characters. It defaults to ".'))
    dialect_doublequote = forms.BooleanField(label=_('Doublequote'), required=False, help_text=_(u'Controls how instances of the quote character appearing inside a field should be themselves be quoted. When True, the character is doubled. When False, the escape character is used as a prefix to the quote character. It defaults to True.'))
    dialect_escapechar = forms.CharField(label=_('Escape character'), max_length=1, required=False, help_text=_(u'The escape character removes any special meaning from the following character. It defaults to None, which disables escaping.'))
    dialect_skipinitialspace = forms.BooleanField(label=_('Skip initial space'), required=False, help_text=_(u'When True, whitespace immediately following the delimiter is ignored. The default is False.'))


class ExpressionForm(forms.Form):
    model_field = forms.CharField(label=_(u'Model field'))
    expression = forms.CharField(label=_(u'Expression'))
    arguments = forms.CharField(label=_(u'Arguments'), required=False)
    enabled = forms.BooleanField(label=_(u'Enabled'), required=False)


class ImportResultForm(forms.Form):
    def __init__(self, *args, **kwargs):
        title = _(u'import results')
        if 'initial' in kwargs:
            if kwargs['initial']:
                title = kwargs['initial'].pop('title', _(u'import results'))
        
        self.title = title
        super(ImportResultForm, self).__init__(*args, **kwargs)
        
    result_area = forms.CharField(label=_(u'Results'), required=False, widget=forms.widgets.Textarea(attrs={'cols':80, 'rows':10}))
    

class ImportWizard(BoundFormWizard):
    def store_settings(self, request, settings_list):
        request.session['last_import_settings'] = reduce_dict(self.settings, settings_list)

    def sniff_file(self):
        csvfile = open(self.settings['filename'], 'rb')
        dialect = csv.Sniffer().sniff(csvfile.read(1024))
        csvfile.seek(0)
        self.initial = {0:{
                'preview_area':''.join([csvfile.readline() for i in range(10)]),
                'dialect_delimiter':dialect.delimiter,
                'dialect_doublequote':dialect.doublequote,
                'dialect_escapechar':dialect.escapechar,
                'dialect_quotechar':dialect.quotechar,
                'dialect_skipinitialspace':dialect.skipinitialspace,
            }}
        csvfile.close()

    def render_template(self, request, form, previous_fields, step, context=None):
        context = {'step_title':self.extra_context['step_titles'][step]}
        return super(ImportWizard, self).render_template(request, form, previous_fields, step, context)
       
    def parse_params(self, request, *args, **kwargs):
        self.extra_context={'step_titles':[
            _(u'step 1 of 4: Import preview'),
            _(u'step 2 of 4: expressions'),
            _(u'step 3 of 4: Import test run results'),
            _(u'step 4 of 4: Final import results'),
            ]}
        self.settings = {}
        self.settings['filename'] = request.GET.get('temp_file', None)
        self.settings['model_name'] = request.GET.get('model_name', None)
        if not self.settings['filename']:
            raise Http404
            
        try:
            self.sniff_file()
        except IOError, err:
            raise Http404(err)
        
    def get_template(self, step):
        return 'import_wizard.html'
    
       
    def process_step(self, request, form, step):
        if step == 0:
            self.settings['dialect_settings'] = dict([(key, form.cleaned_data[key]) for key in form.cleaned_data if 'dialect' in key])
            self.settings['start_row'] = form.cleaned_data['start_row']
            #app_label, name = self.settings['model_name'].split('.')
            app_label, model = self.settings['model_name'].split('.')
            #ct = ContentType.objects.get(app_label=app_label, name=name)
            ct = ContentType.objects.get(app_label=app_label, model=model)
            self.settings['model'] = ct.model_class()
            initial=[]
            for num, field in enumerate(self.settings['model']._meta.fields):
                if field.name != 'id':
                    initial.append({
                        'model_field':field.name,
                        'expression':'"%%s" %% csv_column[%s]' % str(num-1),
                        'enabled':field.name != 'id' and True
                        })
            self.initial = {1:initial}
        elif step == 1:
            self.settings['expressions'] = form.cleaned_data
            results = perform_import(self.settings['filename'], self.settings['model'], self.settings['expressions'], dialect_settings=self.settings['dialect_settings'], start_row=self.settings['start_row'], dryrun=True)

            self.initial = {2:
                {'result_area':'\n'.join(results),
                'title':_(u'Import test run results')}
            }
        elif step == 2:
            self.store_settings(request, ['model_name', 'start_row', 'expressions', 'dialect_settings'])
            results = perform_import(self.settings['filename'], self.settings['model'], self.settings['expressions'], dialect_settings=self.settings['dialect_settings'], start_row=self.settings['start_row'], dryrun=False)
            
            self.initial = {3:
                {'result_area':'\n'.join(results),
                'title':_(u'Final import results')}
            }
           
   
    def done(self, request, form_list):
        os.unlink(self.settings['filename'])
        return HttpResponseRedirect('/')



from django.conf import settings
from django.conf.urls import include, patterns, url
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    # ----Django
    (r'^admin/', include(admin.site.urls)),

    #(r'^i18n/', include('django.conf.urls.i18n')),

    # ----Project
    (r'^', include('main.urls')),
    (r'^common/', include('common.urls')),
    (r'^inventory/', include('inventory.urls')),
    (r'^assets/', include('assets.urls')),
    (r'^search/', include('dynamic_search.urls')),
    #(r'^import/', include('importer.urls')),
    (r'^movements/', include('movements.urls')),
    (r'^generic_photos/', include('photos.urls')),
)

if settings.DEBUG:
    from django.conf.urls.static import static
    urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

    if 'rosetta' in settings.INSTALLED_APPS:
        urlpatterns += patterns('',
            url(r'^rosetta/', include('rosetta.urls'), name='rosetta'),
        )



import types

from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch
from django.core.urlresolvers import RegexURLResolver, RegexURLPattern, Resolver404, get_resolver
from django.template import TemplateSyntaxError, Library, \
                            VariableDoesNotExist, Node, Variable
from django.utils.text import unescape_string_literal

#from common.api import object_navigation
from web_theme.conf import settings as web_theme_settings

register = Library()


class GetThemeNode(Node):
    def __init__(self, var_name, *args):
        self.var_name = var_name

    def render(self, context):
        context['web_theme'] = web_theme_settings.THEME
        return ''


import re

@register.tag
def get_theme(parser, token):
    try:
        # Splitting by None == splitting by spaces.
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires arguments" % token.contents.split()[0]

    #m = re.search(r'(.*?) as (\w+)', arg)
    m = re.search(r'as (\w+)', arg)
    if not m:
        raise template.TemplateSyntaxError, "%r tag had invalid arguments" % tag_name
    #format_string, var_name = m.groups()
    var_name = m.groups()
    
    #if not (format_string[0] == format_string[-1] and format_string[0] in ('"', "'")):
    #    raise template.TemplateSyntaxError, "%r tag's argument should be in quotes" % tag_name
    return GetThemeNode(var_name)    



from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _

from common.api import register_links, register_menu

from .models import State, Person, Item, ItemGroup

state_list = {'text': _('Assets states'), 'view': 'state_list', 'famfam': 'error_go'}
state_create = {'text': _('Create new asset state'), 'view': 'state_create', 'famfam': 'error_add'}
state_edit = {'text': _(u'Edit asset state'), 'view': 'state_update', 'args': 'object.id', 'famfam': 'error'}
state_delete = {'text': _(u'Delete asset state'), 'view': 'state_delete', 'args': 'object.id', 'famfam': 'error_delete'}

person_list = {'text': _('View all people'), 'view': 'person_list', 'famfam': 'user_go'}
person_create = {'text': _('Create new person'), 'view': 'person_create', 'famfam': 'user_add'}
person_update = {'text': _(u'Edit'), 'view': 'person_update', 'args': 'object.id', 'famfam': 'user_edit'}
person_delete = {'text': _(u'Delete'), 'view': 'person_delete', 'args': 'object.id', 'famfam': 'user_delete'}
person_photos = {'text': _(u'Add / remove photos'), 'view': 'person_photos', 'args': 'object.id', 'famfam': 'picture_edit'}
person_assign_item = {'text': _(u'Assign assets'), 'view': 'person_assign_item', 'args': 'object.id', 'famfam': 'computer_go'}

asset_list = {'text': _('View all assets'), 'view': 'item_list', 'famfam': 'computer'}
asset_create = {'text': _('Create new asset'), 'view': 'item_create', 'famfam': 'computer_add'}
asset_orphan_list = {'text': _('Orphan assets'), 'view': 'item_orphans_list'}
asset_edit = {'text': _(u'Edit'), 'view': 'item_update', 'args': 'object.id', 'famfam': 'computer_edit'}
asset_delete = {'text': _(u'Delete'), 'view': 'item_delete', 'args': 'object.id', 'famfam': 'computer_delete'}
asset_photos = {'text': _(u'Add / remove photos'), 'view': 'item_photos', 'args': 'object.id', 'famfam': 'picture_edit'}
asset_assign_person = {'text': _(u'Assign people'), 'view': 'item_assign_person', 'args': 'object.id', 'famfam': 'user_go'}
asset_template = {'text': _(u'Template'), 'view': 'template_view', 'args': 'object.item_template.id', 'famfam': 'page_go'}

group_list = {'text': _(u'View all item groups'), 'view': 'group_list', 'famfam': 'chart_pie'}
group_create = {'text': _(u'Create item group'), 'view': 'group_create', 'famfam': 'chart_pie_add'}
group_update = {'text': _(u'Edit'), 'view': 'group_update', 'args': 'object.id', 'famfam': 'chart_pie_edit'}
group_delete = {'text': _(u'Delete'), 'view': 'group_delete', 'args': 'object.id', 'famfam': 'chart_pie_delete'}

state_filter = {'name': 'state', 'title': _(u'State'), 'queryset': State.objects.all(), 'destination': 'itemstate'}

register_links(['item_list', 'item_view', 'item_create', 'item_orphans_list', 'item_update', 'item_delete', 'item_photos', 'item_assign_person', 'template_items_list'], [asset_create], menu_name='sidebar')
register_links(Item, [asset_edit, asset_delete, asset_photos, asset_assign_person, asset_template])

register_links(['person_list', 'person_create', 'person_view', 'person_update', 'person_delete', 'person_photos', 'person_assign_item'], [person_create], menu_name='sidebar')
register_links(Person, [person_update, person_delete, person_photos, person_assign_item])

register_links(['group_list', 'group_view', 'group_create', 'group_update', 'group_delete'], [group_create], menu_name='sidebar')
register_links(ItemGroup, [group_update, group_delete])

register_links(['state_list', 'state_create', 'state_update', 'state_delete'], [state_create], menu_name='sidebar')
register_links(State, [state_edit, state_delete])

register_menu([
    {'text': _('assets'), 'view': 'item_list', 'links': [
        asset_list, asset_orphan_list, group_list, person_list
    ], 'famfam': 'computer', 'position': 2},
])



from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _

from common.api import register_links, register_menu

from .models import (PurchaseRequestStatus, PurchaseRequest,
                     PurchaseRequestItem, PurchaseOrderStatus,
                     PurchaseOrderItemStatus, PurchaseOrder, PurchaseOrderItem)

purchase_request_state_list = {'text': _('Purchase request states'), 'view': 'purchase_request_state_list', 'famfam': 'pencil_go'}
purchase_request_state_create = {'text': _('Create new purchase request state'), 'view': 'purchase_request_state_create', 'famfam': 'pencil_add'}
purchase_request_state_update = {'text': _('Edit state'), 'view': 'purchase_request_state_update', 'args': 'object.id', 'famfam': 'pencil'}
purchase_request_state_delete = {'text': _('Delete state'), 'view': 'purchase_request_state_delete', 'args': 'object.id', 'famfam': 'pencil_delete'}

purchase_request_list = {'text': _('Purchase requests'), 'view': 'purchase_request_list', 'famfam': 'basket_go'}
purchase_request_create = {'text': _('Create new request'), 'view': 'purchase_request_create', 'famfam': 'basket_add'}
purchase_request_update = {'text': _('Edit request'), 'view': 'purchase_request_update', 'args': 'object.id', 'famfam': 'basket_edit'}
purchase_request_delete = {'text': _('Delete request'), 'view': 'purchase_request_delete', 'args': 'object.id', 'famfam': 'basket_delete'}
purchase_request_close = {'text': _('Close request'), 'view': 'purchase_request_close', 'args': 'object.id', 'famfam': 'cross'}
purchase_request_open = {'text': _('Open request'), 'view': 'purchase_request_open', 'args': 'object.id', 'famfam': 'accept'}
purchase_request_po_wizard = {'text': _('Purchase order wizard'), 'view': 'purchase_order_wizard', 'args': 'object.id', 'famfam': 'wand'}

purchase_request_item_create = {'text': _('Add new item'), 'view': 'purchase_request_item_create', 'args': 'object.id', 'famfam': 'basket_put'}
purchase_request_item_update = {'text': _('Edit item'), 'view': 'purchase_request_item_update', 'args': 'object.id', 'famfam': 'basket_go'}
purchase_request_item_delete = {'text': _('Delete item'), 'view': 'purchase_request_item_delete', 'args': 'object.id', 'famfam': 'basket_remove'}

purchase_order_state_list = {'text': _('Purchase order states'), 'view': 'purchase_order_state_list', 'famfam': 'pencil_go'}
purchase_order_state_create = {'text': _('Create new purchase order state'), 'view': 'purchase_order_state_create', 'famfam': 'pencil_add'}
purchase_order_state_update = {'text': _('Edit state'), 'view': 'purchase_order_state_update', 'args': 'object.id', 'famfam': 'pencil'}
purchase_order_state_delete = {'text': _('Delete state'), 'view': 'purchase_order_state_delete', 'args': 'object.id', 'famfam': 'pencil_delete'}

purchase_order_item_state_list = {'text': _('Purchase order item states'), 'view': 'purchase_order_item_state_list', 'famfam': 'pencil_go'}
purchase_order_item_state_create = {'text': _('Create new item state'), 'view': 'purchase_order_item_state_create', 'famfam': 'pencil_add'}
purchase_order_item_state_update = {'text': _('Edit state'), 'view': 'purchase_order_item_state_update', 'args': 'object.id', 'famfam': 'pencil'}
purchase_order_item_state_delete = {'text': _('Delete state'), 'view': 'purchase_order_item_state_delete', 'args': 'object.id', 'famfam': 'pencil_delete'}

purchase_order_list = {'text': _('Purchase orders'), 'view': 'purchase_order_list', 'famfam': 'cart_go'}
purchase_order_create = {'text': _('Create new order'), 'view': 'purchase_order_create', 'famfam': 'cart_add'}
purchase_order_update = {'text': _('Edit order'), 'view': 'purchase_order_update', 'args': 'object.id', 'famfam': 'cart_edit'}
purchase_order_delete = {'text': _('Delete order'), 'view': 'purchase_order_delete', 'args': 'object.id', 'famfam': 'cart_delete'}
purchase_order_close = {'text': _('Close order'), 'view': 'purchase_order_close', 'args': 'object.id', 'famfam': 'cross'}
purchase_order_open = {'text': _('Open order'), 'view': 'purchase_order_open', 'args': 'object.id', 'famfam': 'accept'}
purchase_order_transfer = {'text': _('Transfer entire order'), 'view': 'purchase_order_transfer', 'args': 'object.id', 'famfam': 'package_link'}

purchase_order_item_create = {'text': _('Add new item'), 'view': 'purchase_order_item_create', 'args': 'object.id', 'famfam': 'cart_put'}
purchase_order_item_update = {'text': _('Edit item'), 'view': 'purchase_order_item_update', 'args': 'object.id', 'famfam': 'cart_go'}
purchase_order_item_delete = {'text': _('Delete item'), 'view': 'purchase_order_item_delete', 'args': 'object.id', 'famfam': 'cart_remove'}
purchase_order_item_close = {'text': _('Close item'), 'view': 'purchase_order_item_close', 'args': 'object.id', 'famfam': 'cross'}
purchase_order_item_transfer = {'text': _('Transfer item'), 'view': 'purchase_order_item_transfer', 'args': 'object.id', 'famfam': 'package_link'}

jump_to_template = {'text': _(u'Template'), 'view': 'template_view', 'args': 'object.item_template.id', 'famfam': 'page_go'}

purchase_request_state_filter = {'name': 'purchase_request_status', 'title': _(u'Status'), 'queryset': PurchaseRequestStatus.objects.all(), 'destination': 'status'}
purchase_order_state_filter = {'name': 'purchase_order_status', 'title': _(u'Status'), 'queryset': PurchaseOrderStatus.objects.all(), 'destination': 'status'}

register_links(PurchaseRequestStatus, [purchase_request_state_update, purchase_request_state_delete])
register_links(['purchase_request_state_create', 'purchase_request_state_list', 'purchase_request_state_update', 'purchase_request_state_delete'], [purchase_request_state_create], menu_name='sidebar')

register_links(PurchaseRequest, [purchase_request_update, purchase_request_delete, purchase_request_item_create, purchase_request_close, purchase_request_open, purchase_request_po_wizard])
register_links(['purchase_request_list', 'purchase_request_create', 'purchase_request_update', 'purchase_request_delete', 'purchase_request_view', 'purchase_order_wizard'], [purchase_request_create], menu_name='sidebar')

register_links(PurchaseRequestItem, [purchase_request_item_update, purchase_request_item_delete, jump_to_template])
register_links(['purchase_request_item_create'], [purchase_request_create], menu_name='sidebar')

register_links(PurchaseOrderStatus, [purchase_order_state_update, purchase_order_state_delete])
register_links(['purchase_order_state_create', 'purchase_order_state_list', 'purchase_order_state_update', 'purchase_order_state_delete'], [purchase_order_state_create], menu_name='sidebar')

register_links(PurchaseOrderItemStatus, [purchase_order_item_state_update, purchase_order_item_state_delete])
register_links(['purchase_order_item_state_create', 'purchase_order_item_state_list', 'purchase_order_item_state_update', 'purchase_order_item_state_delete'], [purchase_order_item_state_create], menu_name='sidebar')

register_links(PurchaseOrder, [purchase_order_update, purchase_order_delete, purchase_order_item_create, purchase_order_close, purchase_order_open, purchase_order_transfer])
register_links(['purchase_order_list', 'purchase_order_create', 'purchase_order_update', 'purchase_order_delete', 'purchase_order_view', 'supplier_purchase_orders'], [purchase_order_create], menu_name='sidebar')

register_links(PurchaseOrderItem, [purchase_order_item_update, purchase_order_item_delete, jump_to_template, purchase_order_item_close, purchase_order_item_transfer])
register_links(['purchase_order_item_create'], [purchase_order_create], menu_name='sidebar')

register_menu([
    {'text': _('Purchases'), 'view': 'purchase_request_list', 'links': [
        purchase_request_list, purchase_order_list,
    ], 'famfam': 'basket', 'position': 4}]
)



search_list = {}

def register(model, text, field_list):
    if model in search_list:
        search_list[model]['fields'].append(field_list)
    else:
        search_list[model] = {'fields':field_list, 'text':text}



from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from photologue.models import ImageModel


class GenericPhotoManager(models.Manager):
    def photos_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        return self.filter(content_type__pk=object_type.id, object_id=obj.id)


class GenericPhoto(ImageModel):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    title = models.CharField(max_length=10, null=True, blank=True, verbose_name=_(u'Title'))
    main = models.BooleanField(default=False, verbose_name=_(u'Main photo?'))

    objects = GenericPhotoManager()

    class Meta:
        verbose_name = _(u'Photo')
        verbose_name_plural = _(u'Photos')

    def __unicode__(self):
        return '%s - %s' % (self.content_object, self.title)



from __future__ import absolute_import

import datetime

from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse, reverse_lazy
from django.forms.formsets import formset_factory
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.utils.translation import ugettext as _

from common.utils import encapsulate
from generic_views.views import (GenericCreateView, GenericDeleteView,
                                 GenericListView, GenericUpdateView)
from inventory.models import Supplier, ItemTemplate, InventoryTransaction

from . import purchase_order_state_filter, purchase_request_state_filter
from .forms import (PurchaseOrderForm, PurchaseOrderForm_view,
                    PurchaseOrderItem, PurchaseOrderItemForm,
                    PurchaseOrderItemTransferForm, PurchaseOrderWizardItemForm,
                    PurchaseRequestForm, PurchaseRequestForm_view,
                    PurchaseRequestItemForm)
from .models import (PurchaseOrder, PurchaseOrderItemStatus,
                     PurchaseOrderStatus, PurchaseRequest,
                     PurchaseRequestItem, PurchaseRequestStatus)


class PurchaseOrderItemDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase order item')}
    model = PurchaseOrderItem

    def get_success_url(self):
        return self.get_object().purchase_order.get_absolute_url()


class PurchaseOrderItemUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Purchase order item')}
    form_class = PurchaseOrderItemForm
    model = PurchaseOrderItem


class PurchaseOrderItemStatusCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Purchase order item state')}
    model = PurchaseOrderItemStatus


class PurchaseOrderItemStatusDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase order item state')}
    model = PurchaseOrderItemStatus
    success_url = reverse_lazy('purchase_order_item_state_list')


class PurchaseOrderItemStatusListView(GenericListView):
    extra_context = {'title': _(u'Purchase order item status')}
    model = PurchaseOrderItemStatus


class PurchaseOrderItemStatusUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Purchase order item state')}
    model = PurchaseOrderItemStatus


class PurchaseOrderCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Purchase order')}
    form_class = PurchaseOrderForm


class PurchaseOrderDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase order')}
    model = PurchaseOrder
    success_url = reverse_lazy('purchase_order_list')


class PurchaseOrderListView(GenericListView):
    extra_context = {
        'title': _(u'Purchase orders'),
        'extra_columns': [
            {'name': _(u'Active'), 'attribute': encapsulate(lambda x: _(u'Open') if x.active else _(u'Closed'))}
        ]
    }
    list_filters = [purchase_order_state_filter]
    model = PurchaseOrder


class PurchaseOrderUpdateView(GenericUpdateView):
    form_class = PurchaseOrderForm
    model = PurchaseOrder


class PurchaseOrderStatusCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Purchase order status')}
    model = PurchaseOrderStatus


class PurchaseOrderStatusDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase order status')}
    model = PurchaseOrderStatus
    success_url = reverse_lazy('purchase_order_state_list')


class PurchaseOrderStatusListView(GenericListView):
    extra_context = {'title': _(u'Purchase order status')}
    model = PurchaseOrderStatus


class PurchaseOrderStatusUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Purchase order status')}
    model = PurchaseOrderStatus


class PurchaseRequestCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Purchase request')}
    form_class = PurchaseRequestForm


class PurchaseRequestDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase request')}
    model = PurchaseRequest
    success_url = reverse_lazy('purchase_request_list')


class PurchaseRequestListView(GenericListView):
    model = PurchaseRequest
    extra_context = {
        'title': _(u'Purchase requests'),
        'extra_columns': [
            {'name': _(u'Active'), 'attribute': encapsulate(lambda x: _(u'Open') if x.active else _(u'Closed'))}
        ]
    }
    list_filters = [purchase_request_state_filter]


class PurchaseRequestUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Purchase request')}
    form_class = PurchaseRequestForm
    model = PurchaseRequest


class PurchaseRequestStateCreateView(GenericCreateView):
    extra_context = {'object_name': _(u'Purchase request state')}
    model = PurchaseRequestStatus


class PurchaseRequestStateDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase request state')}
    model = PurchaseRequestStatus
    success_url = reverse_lazy('purchase_request_state_list')


class PurchaseRequestStateListView(GenericListView):
    extra_context = {'title': _(u'Purchase request states')}
    model = PurchaseRequestStatus


class PurchaseRequestStateUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Purchase request state')}
    model = PurchaseRequestStatus


class PurchaseRequestItemDeleteView(GenericDeleteView):
    extra_context = {'object_name': _(u'Purchase request item')}
    model = PurchaseRequestItem

    def get_success_url(self):
        return self.get_object().purchase_request.get_absolute_url()


class PurchaseRequestItemUpdateView(GenericUpdateView):
    extra_context = {'object_name': _(u'Purchase request item')}
    model = PurchaseRequestItem


def purchase_request_view(request, object_id):
    purchase_request = get_object_or_404(PurchaseRequest, pk=object_id)
    form = PurchaseRequestForm_view(
        instance=purchase_request,
        extra_fields=[
            {'field': 'purchaseorder_set.all', 'label': _(u'Related purchase orders')}
        ]
    )

    return render_to_response('generic_views/generic_detail.html', {
        'title': _(u'Details for purchase request: %s') % purchase_request,
        'object': purchase_request,
        'form': form,
        'subtemplates_dict': [
            {
                'name': 'generic_views/generic_list_subtemplate.html',
                'title': _(u'Purchase request items'),
                'object_list': purchase_request.items.all(),
                'extra_columns': [{'name': _(u'Qty'), 'attribute': 'qty'}],
            },
            #TODO: Used this instead when pagination namespace is supported
            #{
            #    'name':'generic_list_subtemplate.html',
            #    'title':_(u'related purchase orders'),
            #    'object_list':purchase_request.purchaseorder_set.all(),
            #    'extra_columns':[{'name':_(u'issue data'), 'attribute':'issue_date'}],
            #}
        ]
    }, context_instance=RequestContext(request))


def purchase_request_item_create(request, object_id):
    purchase_request = get_object_or_404(PurchaseRequest, pk=object_id)

    if request.method == 'POST':
        form = PurchaseRequestItemForm(request.POST)#, initial={'purchase_request':purchase_request})
        if form.is_valid():
            form.save()
            msg = _(u'The purchase request item was created successfully.')
            messages.success(request, msg, fail_silently=True)
            return redirect(purchase_request.get_absolute_url())
    else:
        form = PurchaseRequestItemForm(initial={'purchase_request': purchase_request})

    return render_to_response('generic_views/generic_form.html', {
        'form': form,
        'title': _(u'add new purchase request item'),
    }, context_instance=RequestContext(request))


def purchase_request_close(request, object_id):
    purchase_request = get_object_or_404(PurchaseRequest, pk=object_id)

    data = {
        'object': purchase_request,
        'title': _(u'Are you sure you wish to close the purchase request: %s?') % purchase_request,
    }

    if not purchase_request.active:
        msg = _(u'This purchase request has already been closed.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_request.get_absolute_url())

    if request.method == 'POST':
        purchase_request.active = False
        purchase_request.save()
        msg = _(u'The purchase request has been closed successfully.')
        messages.success(request, msg, fail_silently=True)
        return redirect(purchase_request.get_absolute_url())

    return render_to_response('generic_views/generic_confirm.html', data,
        context_instance=RequestContext(request))


def purchase_request_open(request, object_id):
    purchase_request = get_object_or_404(PurchaseRequest, pk=object_id)

    data = {
        'object': purchase_request,
        'title': _(u'Are you sure you wish to open the purchase request: %s?') % purchase_request,
    }

    if purchase_request.active:
        msg = _(u'This purchase request is already open.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_request.get_absolute_url())

    if request.method == 'POST':
        purchase_request.active = True
        purchase_request.save()
        msg = _(u'The purchase request has been opened successfully.')
        messages.success(request, msg, fail_silently=True)
        return redirect(purchase_request.get_absolute_url())

    return render_to_response('generic_views/generic_confirm.html', data,
        context_instance=RequestContext(request))


def purchase_order_wizard(request, object_id):
    """
    Creates new purchase orders based on the item suppliers selected
    from a purchase request
    """

    purchase_request = get_object_or_404(PurchaseRequest, pk=object_id)

    # A closed purchase orders may also mean a PO has been generated
    # previously from it by this wizard
    if not purchase_request.active:
        msg = _(u'This purchase request is closed.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_request.get_absolute_url())

    if not purchase_request.items.all():
        msg = _(u'This purchase request is empty, add items before using the wizard.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_request.get_absolute_url())

    # Create a formset for all the items in the purchase request
    # and let the user select from the available suppliers from each
    # item
    ItemsFormSet = formset_factory(PurchaseOrderWizardItemForm, extra=0)

    initial = []
    for item in purchase_request.items.all():
        initial.append({
            'item': item
        })

    if request.method == 'POST':
        formset = ItemsFormSet(request.POST, initial=initial)
        if formset.is_valid():
            # Create a dictionary of supplier and corresponding items
            # to be ordered from them
            # TODO: Can this be done with a reduce function?
            suppliers = {}
            for form in formset.forms:
                supplier = get_object_or_404(Supplier, pk=form.cleaned_data['supplier'])
                item_template = get_object_or_404(ItemTemplate, pk=form.cleaned_data['template_id'])
                if supplier in suppliers:
                    suppliers[supplier].append({'item_template': item_template, 'qty': form.cleaned_data['qty']})
                else:
                    suppliers[supplier] = [{'item_template': item_template, 'qty': form.cleaned_data['qty']}]

            # Create a new purchase order for each supplier in the
            # suppliers directory
            new_pos = []
            for supplier, po_items_data in suppliers.items():
                purchase_order = PurchaseOrder(
                    purchase_request=purchase_request,
                    supplier=supplier
                )
                new_pos.append(purchase_order)
                purchase_order.save()

                # Create the purchase order items
                for po_item_data in po_items_data:
                    po_item = PurchaseOrderItem(
                        purchase_order=purchase_order,
                        item_template=po_item_data['item_template'],
                        qty=po_item_data['qty']
                    )
                    po_item.save()

            purchase_request.active = False
            purchase_request.save()
            msg = _(u'The following new purchase order have been created: %s.') % (', '.join(['%s' % po for po in new_pos]))
            messages.success(request, msg, fail_silently=True)

            return redirect('purchase_order_list')
    else:
        formset = ItemsFormSet(initial=initial)
    return render_to_response('generic_views/generic_form.html', {
        'form': formset,
        'form_display_mode_table': True,
        'title': _(u'Purchase order wizard, using purchase request source: <a href="%(url)s">%(name)s</a>') % {'url': purchase_request.get_absolute_url(), 'name': purchase_request},
        'object': purchase_request,
    }, context_instance=RequestContext(request))


def purchase_order_view(request, object_id):
    purchase_order = get_object_or_404(PurchaseOrder, pk=object_id)
    form = PurchaseOrderForm_view(instance=purchase_order)

    return render_to_response('generic_views/generic_detail.html', {
        'title': _(u'Details for purchase order: %s') % purchase_order,
        'object': purchase_order,
        'form': form,
        'subtemplates_dict': [{
            'name': 'generic_views/generic_list_subtemplate.html',
            'title': _(u'Purchase order items'),
            'object_list': purchase_order.items.all(),
            'extra_columns': [
                {'name': _(u'Qty'), 'attribute': 'qty'},
                {'name': _(u'Qty received'), 'attribute': 'received_qty'},
                {'name': _(u'Agreed price'), 'attribute': encapsulate(lambda x: '%s' % x.agreed_price if x.agreed_price else '-')},
                {'name': _(u'Status'), 'attribute': 'status'},
                {'name': _(u'Active'), 'attribute': encapsulate(lambda x: _(u'Open') if x.active else _(u'Closed'))}
            ],
        }]
    }, context_instance=RequestContext(request))


def purchase_order_close(request, object_id):
    purchase_order = get_object_or_404(PurchaseOrder, pk=object_id)
    items = purchase_order.items.all()

    data = {
        'object': purchase_order,
        'title': _(u'Are you sure you wish to close the purchase order: %s?') % purchase_order,
    }

    if items.filter(active=True):
        data['message'] = _(u'There are still open items.')

    if not purchase_order.active:
        msg = _(u'This purchase order has already been closed.')
        messages.error(request, msg, fail_silently=True)
        return redirect(purchase_order.get_absolute_url())

    if request.method == 'POST':
        purchase_order.active = False
        items.update(active=False)
        purchase_order.save()
        msg = _(u'The purchase order has been closed successfully.')
        messages.success(request, msg, fail_silently=True)
        return redirect(purchase_order.get_absolute_url())

    return render_to_response('generic_views/generic_confirm.html', data,
        context_instance=RequestContext(request))


def purchase_order_open(request, object_id):
    purchase_order = get_object_or_404(PurchaseOrder, pk=object_id)

    data = {
        'object': purchase_order,
        'title': _(u'Are you sure you wish to open the purchase order: %s?') % purchase_order,
    }

    if purchase_order.active:
        msg = _(u'This purchase order is already open.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_order.get_absolute_url())

    if request.method == 'POST':
        purchase_order.active = True
        purchase_order.save()
        msg = _(u'The purchase order has been opened successfully.')
        messages.success(request, msg, fail_silently=True)
        return redirect(purchase_order.get_absolute_url())

    return render_to_response('generic_views/generic_confirm.html', data,
        context_instance=RequestContext(request))


def purchase_order_transfer(request, object_id):
    """
    Take a purchase order and call transfer_to_inventory to transfer and
    close all of its item and close the purchase order too
    """

    purchase_order = get_object_or_404(PurchaseOrder, pk=object_id)

    if not purchase_order.active:
        msg = _(u'This purchase order has already been closed.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_order.get_absolute_url())

    return transfer_to_inventory(request, purchase_order)


def transfer_to_inventory(request, object_to_transfer):
    """
    Take an item from a purchase order, or an entire purchase order and
    create inventory transaction entries to add the items to an inventory
    and close the item and/or the purchase order
    """
    FormSet = formset_factory(PurchaseOrderItemTransferForm, extra=0)
    context = {}

    if isinstance(object_to_transfer, PurchaseOrderItem):
        # A single purchase order item
        context = {
            'object_name': _(u'Purchase order item'),
            'title': _(u'Transfer and close the received purchase orders item: %s') % object_to_transfer,
        }
        # Seed a formset of a single form
        initial = [{
            'qty': object_to_transfer.received_qty,
            'purchase_order_item': object_to_transfer,
            'purchase_order_item_id': object_to_transfer.id
        }]
    elif isinstance(object_to_transfer, PurchaseOrder):
        # An entire purchase order
        context = {
            'object_name': _(u'Purchase order'),
        }
        # Seed the formset for each PO item
        initial = []
        items = object_to_transfer.items.filter(active=True)
        if not items:
            # There are no PO items active
            context['title'] = _(u'All the items from this purchase order are either closed or transfered, continue to close the purchase order.')
        else:
            context['title'] = _(u'Transfer and close all the received items from the purchase order: %s') % object_to_transfer

        for item in items:
            initial.append({
                'qty': item.received_qty,
                'purchase_order_item': item,
                'purchase_order_item_id': item.id
            })
    # TODO: Raise error if object_to_transfer is neither a PO nor a PO item?
    # else:
    #    raise 'Unknown object type'

    if request.method == 'POST':
        formset = FormSet(request.POST)
        if formset.is_valid():
            if len(formset.forms):
                # In case of empty PO or PO w/ no active items
                for form in formset.forms:
                    # Create inventory transaction for each PO item
                    purchase_order_item = get_object_or_404(PurchaseOrderItem, pk=form.cleaned_data['purchase_order_item_id'])
                    transaction = InventoryTransaction(
                        inventory=form.cleaned_data['inventory'],
                        supply=purchase_order_item.item_template,
                        quantity=form.cleaned_data['qty'],
                        date=datetime.date.today(),
                        notes=_(u'Automatically transfered from purchase order:%s') % purchase_order_item.purchase_order
                    )
                    transaction.save()
                    purchase_order_item.active = False
                    purchase_order_item.save()

                if isinstance(object_to_transfer, PurchaseOrderItem):
                    msg = _(u'The purchase order item has been transfered and closed successfully.')
                    messages.success(request, msg, fail_silently=True)
                    return redirect(purchase_order_item.get_absolute_url())
                elif isinstance(object_to_transfer, PurchaseOrder):
                    object_to_transfer.active = False
                    object_to_transfer.save()
                    msg = _(u'All the purchase order items have been transfered and closed successfully, the purchase order has been closed as well.')
                    messages.success(request, msg, fail_silently=True)
                    return redirect(object_to_transfer.get_absolute_url())
            else:
                # Empty PO or PO w/ no active items
                object_to_transfer.active = False
                object_to_transfer.save()
                msg = _(u'All the purchase order items were closed or already transfered, closing the purchase order.')
                messages.success(request, msg, fail_silently=True)
                return redirect(object_to_transfer.get_absolute_url())

    else:
        formset = FormSet(initial=initial)

    context.update({
        'object': object_to_transfer,
        'form': formset,
        'form_display_mode_table': True
    })

    return render_to_response('generic_views/generic_form.html', context,
        context_instance=RequestContext(request))


def purchase_order_item_transfer(request, object_id):
    """
    Take a purchase order item and call transfer_to_inventory to
    transfer and close the item
    """

    purchase_order_item = get_object_or_404(PurchaseOrderItem, pk=object_id)

    if not purchase_order_item.active:
        msg = _(u'This purchase order item has already been closed.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_order_item.get_absolute_url())

    return transfer_to_inventory(request, purchase_order_item)


def purchase_order_item_close(request, object_id):
    purchase_order_item = get_object_or_404(PurchaseOrderItem, pk=object_id)
    data = {
        'object': purchase_order_item,
        'title': _(u'Are you sure you wish close the purchase order item: %s') % purchase_order_item,
    }

    if not purchase_order_item.active:
        msg = _(u'This purchase order item has already been closed.')
        messages.error(request, msg, fail_silently=True)
        return redirect(request.META['HTTP_REFERER'] if 'HTTP_REFERER' in request.META else purchase_order.get_absolute_url())

    if request.method == 'POST':
        purchase_order_item.active = False
        purchase_order_item.save()
        msg = _(u'The purchase order item has been closed successfully.')
        messages.success(request, msg, fail_silently=True)
        return redirect(purchase_order_item.get_absolute_url())

    return render_to_response('generic_views/generic_confirm.html', data,
        context_instance=RequestContext(request))


def purchase_order_item_create(request, object_id):
    purchase_order = get_object_or_404(PurchaseOrder, pk=object_id)

    if request.method == 'POST':
        form = PurchaseOrderItemForm(request.POST)
        if form.is_valid():
            form.save()
            msg = _(u'The purchase order item was created successfully.')
            messages.success(request, msg, fail_silently=True)
            return redirect(purchase_order.get_absolute_url())
    else:
        form = PurchaseOrderItemForm(initial={'purchase_order': purchase_order})

    return render_to_response('generic_views/generic_form.html', {
        'form': form,
        'title': _(u'Add new purchase order item'),
    }, context_instance=RequestContext(request))



from django.conf.urls import patterns, url

urlpatterns = patterns('photos.views',
    url(r'^(?P<object_id>\d+)/delete/$', 'generic_photo_delete', name='generic_photo_delete'),
    url(r'^(?P<object_id>\d+)/mark_as_main/$', 'generic_photo_mark_main', name='generic_photo_mark_main')
)



import os
import csv
import tempfile

from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.forms.formsets import formset_factory
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.utils.simplejson import dumps, loads
from django.utils.http import urlencode


from forms import DocumentForm, PreviewForm, ExpressionForm, ImportResultForm, ImportWizard

def handle_uploaded_file(f):
    destination, filepath = tempfile.mkstemp()

    destination = open(filepath, 'wb')
    for chunk in f.chunks():
        destination.write(chunk)
    destination.close()
    return filepath
   
   
def import_wizard(request):
    ExpressionFormSet = formset_factory(ExpressionForm, extra=0)
    ExpressionFormSet.title = _(u'expressions')
    wizard = ImportWizard(form_list=[PreviewForm, ExpressionFormSet, ImportResultForm, ImportResultForm])
    return wizard(request)
    
   
def import_file(request):
    models = [
        'inventory.inventory',
        'inventory.inventorytransaction',
        'assets.itemgroup',
        'assets.itemstate',
        'inventory.itemtemplate', 
        'inventory.location',
        'assets.person',
        'assets.state',
        'inventory.supplier',
        'assets.item',
    ]
    if request.method == 'POST':
        form = DocumentForm(request.POST, request.FILES, models=models)
        if form.is_valid():
            temp_file = handle_uploaded_file(form.cleaned_data['local_document'])
            return HttpResponseRedirect(
                '?'.join(
                    [reverse('import_next_steps'),
                     urlencode({
                        'temp_file':temp_file,
                        'model_name':form.cleaned_data['model_name'],
                    })]))
         
    else:
        form = DocumentForm(models=models)


    return render_to_response('generic_form.html', {
        'form':form,
        'title':_(u'Upload a file to import'),
    },
    context_instance=RequestContext(request))    
    
    
def download_last_settings(request):
    settings = request.session.get('last_import_settings', None)
    if settings:
        content=dumps(settings)
        response = HttpResponse(content, content_type='text/plain', mimetype='application/json')
        response['Content-Length'] = len(content)
        response['Content-Disposition'] = "attachment; filename=last_import_settings.json"
        return response
                       

    messages.error(request, _(u'There are no settings available to download.'))
    return HttpResponseRedirect('/')



import types

from django import forms
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from django.db import models


def return_attrib(obj, attrib, arguments=None):
    try:
        result = reduce(getattr, attrib.split('.'), obj)
        if isinstance(result, types.MethodType):
            if arguments:
                return result(**arguments)
            else:
                return result()
        else:
            return result
    except Exception, err:
        if settings.DEBUG:
            return 'Attribute error: %s; %s' % (attrib, err)
        else:
            pass


class DetailSelectMultiple(forms.widgets.SelectMultiple):
    def __init__(self, queryset=None, *args, **kwargs):
        self.queryset=queryset
        super(DetailSelectMultiple, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None, choices=()):
        if value is None: value = ''
        #final_attrs = self.build_attrs(attrs, name=name)
        output = u'<ul class="list">'
        options = None
        if value:
            if getattr(value, '__iter__', None):
                options = [(index, string) for index, string in self.choices if index in value]
            else:
                options = [(index, string) for index, string in self.choices if index == value]
        else:
            if self.choices:
                if self.choices[0] != (u'', u'---------') and value != []:
                    options = [(index, string) for index, string in self.choices]

        if options:
            for index, string in options:
                if self.queryset:
                    try:
                        output += u'<li><a href="%s">%s</a></li>' % (self.queryset.get(pk=index).get_absolute_url(), string)
                    except AttributeError:
                        output += u'<li>%s</li>' % (string)
                else:
                 output += u'<li>%s</li>' % string
        else:
            output += u'<li>%s</li>' % _(u"None")
        return mark_safe(output + u'</ul>\n')


class DetailForm(forms.ModelForm):
    def __init__(self, extra_fields=None, *args, **kwargs):
        super(DetailForm, self).__init__(*args, **kwargs)
        if extra_fields:
            for extra_field in extra_fields:
                result = return_attrib(self.instance, extra_field['field'])
                label = 'label' in extra_field and extra_field['label'] or None
                #TODO: Add others result types <=> Field types
                if isinstance(result, models.query.QuerySet):
                    self.fields[extra_field['field']]=forms.ModelMultipleChoiceField(queryset=result, label=label)

        for field_name, field in self.fields.items():
            if isinstance(field.widget, forms.widgets.SelectMultiple):
                self.fields[field_name].widget = DetailSelectMultiple(
                    choices=field.widget.choices,
                    attrs=field.widget.attrs,
                    queryset=getattr(field, 'queryset', None),
                )
                self.fields[field_name].help_text=''
            elif isinstance(field.widget, forms.widgets.Select):
                self.fields[field_name].widget = DetailSelectMultiple(
                    choices=field.widget.choices,
                    attrs=field.widget.attrs,
                    queryset=getattr(field, 'queryset', None),
                )
                self.fields[field_name].help_text=''


class GenericConfirmForm(forms.Form):
    pass


class GenericAssignRemoveForm(forms.Form):
    left_list = forms.ModelMultipleChoiceField(required=False, queryset=None)
    right_list = forms.ModelMultipleChoiceField(required=False, queryset=None)
    def __init__(self, left_list_qryset=None, right_list_qryset=None, left_filter=None, *args, **kwargs):
        super(GenericAssignRemoveForm, self).__init__(*args, **kwargs)
        if left_filter:
            self.fields['left_list'].queryset = left_list_qryset.filter(*left_filter)
        else:
            self.fields['left_list'].queryset = left_list_qryset

        self.fields['right_list'].queryset = right_list_qryset


class FilterForm(forms.Form):
    def __init__(self, list_filters, *args, **kwargs):
        super(FilterForm, self).__init__(*args, **kwargs)
        for list_filter in list_filters:
            label = list_filter.get('title', list_filter['name'])
            self.fields[list_filter['name']] = forms.ModelChoiceField(queryset=list_filter['queryset'], label=label[0].upper() + label[1:], required=False)



from __future__ import absolute_import

import urllib

from django.core.urlresolvers import reverse, NoReverseMatch
from django.contrib import messages
from django.db.models import Q
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.views.generic import (CreateView, DeleteView, DetailView, ListView,
                                  UpdateView)

from .forms import (FilterForm, GenericConfirmForm, GenericAssignRemoveForm,
                    DetailForm)


class ExtraContextMixin(object):
    extra_context = {}

    def get_context_data(self, **kwargs):
        context = super(ExtraContextMixin, self).get_context_data(**kwargs)
        context.update(self.extra_context)
        return context


class GenericCreateView(ExtraContextMixin, CreateView):
    template_name = 'generic_views/generic_form.html'


class GenericDeleteView(ExtraContextMixin, DeleteView):
    template_name='generic_views/generic_confirm.html'

    def get_context_data(self, **kwargs):
        context = super(GenericDeleteView, self).get_context_data(**kwargs)
        context['delete_view'] = True
        return context

    #kwargs['post_delete_redirect'] = reverse(kwargs['post_delete_redirect'])


class GenericDetailView(ExtraContextMixin, DetailView):
    template_name = 'generic_views/generic_detail.html'

    def get_context_data(self, **kwargs):
        context = super(GenericDetailView, self).get_context_data(**kwargs)
        if hasattr(self, 'extra_fields'):
            context['form'] = self.form_class(instance=self.get_object(), extra_fields=getattr(self, 'extra_fields', None))
        else:
            context['form'] = self.form_class(instance=self.get_object())

        return context


class GenericListView(ExtraContextMixin, ListView):
    filter_form = None
    filters = None
    list_filters = None
    template_name = 'generic_views/generic_list.html'

    def get(self, request, *args, **kwargs):
        if self.list_filters:
            self.filter_form, self.filters = add_filter(self.request, self.list_filters)
        return super(GenericListView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(GenericListView, self).get_context_data(**kwargs)

        if self.filter_form:
            context['filter_form'] = self.filter_form

        return context

    def get_queryset(self):
        queryset = super(GenericListView, self).get_queryset()
        if self.filters:
            return queryset.filter(*self.filters)
        else:
            return queryset


class GenericUpdateView(ExtraContextMixin, UpdateView):
    template_name = 'generic_views/generic_form.html'


def add_filter(request, list_filters):
    filters = []
    filter_dict = dict([(f['name'], f) for f in list_filters])
    if request.method == 'GET':
        filter_form = FilterForm(list_filters, request.GET)
        if filter_form.is_valid():
            for name, data in filter_form.cleaned_data.items():
                if data:
                    filters.append(Q(**{filter_dict[name]['destination']: data}))

    else:
        filter_form = FilterForm(list_filters)

    return filter_form, filters


def generic_assign_remove(request, title, obj, left_list_qryset, left_list_title, right_list_qryset, right_list_title, add_method, remove_method, item_name, list_filter=None):
    left_filter = None
    filter_form = None
    if list_filter:
        filter_form, filters = add_filter(request, list_filter)
        if filters:
            left_filter = filters

    if request.method == 'POST':
        post_data = request.POST
        form = GenericAssignRemoveForm(left_list_qryset, right_list_qryset, left_filter, request.POST)
        if form.is_valid():
            action = post_data.get('action','')
            if action == 'assign':
                for item in form.cleaned_data['left_list']:
                    add_method(item)
                if form.cleaned_data['left_list']:
                    messages.success(request, _(u'The selected %s were added.') % unicode(item_name))

            if action == 'remove':
                for item in form.cleaned_data['right_list']:
                    remove_method(item)
                if form.cleaned_data['right_list']:
                    messages.success(request, _(u'The selected %s were removed.') % unicode(item_name))

    form = GenericAssignRemoveForm(left_list_qryset=left_list_qryset, right_list_qryset=right_list_qryset, left_filter=left_filter)

    return render_to_response('generic_views/generic_assign_remove.html', {
    'form': form,
    'object': obj,
    'title': title,
    'left_list_title': left_list_title,
    'right_list_title': right_list_title,
    'filter_form': filter_form,
    },
    context_instance=RequestContext(request))



from __future__ import absolute_import

from django.conf.urls import patterns, url
from django.utils.translation import ugettext_lazy as _

from generic_views.views import generic_assign_remove
from inventory import location_filter
from photos.views import generic_photos

from . import state_filter
from .conf import settings as asset_settings
from .forms import (ItemForm, ItemForm_view, ItemGroupForm, ItemGroupForm_view,
                    PersonForm, PersonForm_view)
from .models import Item, ItemGroup, Person, State
from .views import (AssetCreateView, AssetDeleteView, AssetDetailView, AssetListView,
                    AssetOrphanListView, AssetUpdateView, GroupCreateView,
                    GroupDeleteView, GroupDetailView, GroupListView,
                    PersonCreateView, PersonDeleteView, PersonDetailView,
                    PersonListView, PersonUpdateView, StateCreateView,
                    StateDeleteView, StateListView, StateUpdateView)

urlpatterns = patterns('assets.views',
    url(r'^person/(?P<object_id>\d+)/photos/$', generic_photos, {'model': Person, 'max_photos': asset_settings.MAX_PERSON_PHOTOS, 'extra_context': {'object_name': _(u'person')}}, 'person_photos'),
    url(r'^person/(?P<pk>\d+)/$', PersonDetailView.as_view(), name='person_view'),
    url(r'^person/list/$', PersonListView.as_view(), name='person_list'),
    url(r'^person/create/$', PersonCreateView.as_view(), name='person_create'),
    url(r'^person/(?P<pk>\d+)/update/$', PersonUpdateView.as_view(), name='person_update'),
    url(r'^person/(?P<pk>\d+)/delete/$', PersonDeleteView.as_view(), name='person_delete'),
    url(r'^person/(?P<object_id>\d+)/assign/$', 'person_assign_remove_item', (), 'person_assign_item'),

    url(r'^asset/create/$', AssetCreateView.as_view(), name='item_create'),
    url(r'^asset/(?P<pk>\d+)/update/$', AssetUpdateView.as_view(), name='item_update'),
    url(r'^asset/(?P<pk>\d+)/delete/$', AssetDeleteView.as_view(), name='item_delete'),
    url(r'^asset/(?P<object_id>\d+)/assign/$', 'item_assign_remove_person', (), name='item_assign_person'),
    url(r'^asset/orphans/$', AssetOrphanListView.as_view(), name='item_orphans_list'),
    url(r'^asset/list/$', AssetListView.as_view(), name='item_list'),
    url(r'^asset/(?P<pk>\d+)/$', AssetDetailView.as_view(), name='item_view'),
    url(r'^asset/(?P<object_id>\d+)/photos/$', generic_photos, {'model': Item, 'max_photos': asset_settings.MAX_ASSET_PHOTOS, 'extra_context': {'object_name': _(u'asset')}}, 'item_photos'),
    url(r'^asset/(?P<object_id>\d+)/state/(?P<state_id>\d+)/set/$', 'item_setstate', (), 'item_setstate'),
    url(r'^asset/(?P<object_id>\d+)/state/(?P<state_id>\d+)/unset$', 'item_remove_state', (), 'item_remove_state'),

    url(r'^group/list/$', GroupListView.as_view(), name='group_list'),
    url(r'^group/create/$', GroupCreateView.as_view(), name='group_create'),
    url(r'^group/(?P<pk>\d+)/$', GroupDetailView.as_view(), name='group_view'),
    url(r'^group/(?P<object_id>\d+)/update/$', 'group_assign_remove_item', (), name='group_update'),
    url(r'^group/(?P<pk>\d+)/delete/$', GroupDeleteView.as_view(), name='group_delete'),

    url(r'^state/list/$', StateListView.as_view(), name='state_list'),
    url(r'^state/create/$', StateCreateView.as_view(), name='state_create'),
    url(r'^state/(?P<pk>\d+)/update/$', StateUpdateView.as_view(), name='state_update'),
    url(r'^state/(?P<pk>\d+)/delete/$', StateDeleteView.as_view(), name='state_delete'),
)



from __future__ import absolute_import

from django.contrib import admin

from .models import Inventory, ItemTemplate, Location, Log, Supplier


class ItemTemplateAdmin(admin.ModelAdmin):
    filter_horizontal = ('supplies', 'suppliers')


admin.site.register(Location)
admin.site.register(Log)
admin.site.register(ItemTemplate, ItemTemplateAdmin)
admin.site.register(Inventory)
admin.site.register(Supplier)



from __future__ import absolute_import

import types

from django.conf import settings
from django.core.urlresolvers import resolve, reverse, NoReverseMatch
from django.core.urlresolvers import (RegexURLResolver, RegexURLPattern,
                                      Resolver404, get_resolver)
from django.template import (TemplateSyntaxError, Library,
                             VariableDoesNotExist, Node, Variable)
from django.utils.text import unescape_string_literal

from ..api import object_navigation, menu_links as menu_navigation

register = Library()


def process_links(links, view_name, url):
    items = []
    active_item = None
    for item, count in zip(links, range(len(links))):
        item_view = 'view' in item and item['view']
        item_url = 'url' in item and item['url']
        if view_name == item_view or url == item_url:
            active = True
            active_item = item
        else:
            active = False
            if 'links' in item:
                for child_link in item['links']:
                    child_view = 'view' in child_link and child_link['view']
                    child_url = 'url' in child_link and child_link['url']
                    if view_name == child_view or url == child_url:
                        active = True
                        active_item = item

        items.append(
            {
                'first': count == 0,
                'active': active,
                'url': item_view and reverse(item_view) or item_url or '#',
                'text': unicode(item['text']),
                'famfam': 'famfam' in item and item['famfam'],
            }
        )
    return items, active_item


class NavigationNode(Node):
    def __init__(self, navigation, *args, **kwargs):
        self.navigation = navigation

    def render(self, context):
        request = Variable('request').resolve(context)
        view_name = resolve(request.META['PATH_INFO']).view_name

        main_items, active_item = process_links(links=self.navigation, view_name=view_name, url=request.META['PATH_INFO'])
        context['navigation_main_links'] = main_items
        if active_item and 'links' in active_item:
            secondary_links, active_item = process_links(links=active_item['links'], view_name=view_name, url=request.META['PATH_INFO'])
            context['navigation_secondary_links'] = secondary_links
        return ''


@register.tag
def main_navigation(parser, token):
    args = token.split_contents()
    return NavigationNode(navigation=menu_navigation)


def resolve_arguments(context, src_args):
    args = []
    kwargs = {}
    if type(src_args) == type([]):
        for i in src_args:
            val = resolve_template_variable(context, i)
            if val:
                args.append(val)
    elif type(src_args) == type({}):
        for key, value in src_args.items():
            val = resolve_template_variable(context, value)
            if val:
                kwargs[key] = val
    else:
        val = resolve_template_variable(context, src_args)
        if val:
            args.append(val)

    return args, kwargs


def resolve_links(context, links, current_view, current_path):
    context_links = []
    for link in links:
        args, kwargs = resolve_arguments(context, link.get('args', {}))

        if 'view' in link:
            link['active'] = link['view'] == current_view
            args, kwargs = resolve_arguments(context, link.get('args', {}))

            try:
                if kwargs:
                    link['url'] = reverse(link['view'], kwargs=kwargs)
                else:
                    link['url'] = reverse(link['view'], args=args)
            except NoReverseMatch, err:
                link['url'] = '#'
                link['error'] = err
        elif 'url' in link:
            link['active'] = link['url'] == current_path
        else:
            link['active'] = False
        context_links.append(link)

    return context_links


def _get_object_navigation_links(context, menu_name=None):
    current_path = Variable('request').resolve(context).META['PATH_INFO']
    current_view = resolve(current_path).view_name
    context_links = []

    try:
        object_name = Variable('navigation_object_name').resolve(context)
    except VariableDoesNotExist:
        object_name = 'object'

    try:
        obj = Variable(object_name).resolve(context)
    except VariableDoesNotExist:
        obj = None

    try:
        links = object_navigation[menu_name][current_view]['links']
        for link in resolve_links(context, links, current_view, current_path):
            context_links.append(link)
    except KeyError:
        pass

    try:
        links = object_navigation[menu_name][type(obj)]['links']
        for link in resolve_links(context, links, current_view, current_path):
            context_links.append(link)
    except KeyError:
        pass

    return context_links


def resolve_template_variable(context, name):
    try:
        return unescape_string_literal(name)
    except ValueError:
        return Variable(name).resolve(context)
    except TypeError:
        return name


class GetNavigationLinks(Node):
    def __init__(self, *args):
        self.menu_name = None
        if args:
            self.menu_name = args[0]

    def render(self, context):
        menu_name = resolve_template_variable(context, self.menu_name)
        context['object_navigation_links'] = _get_object_navigation_links(context, menu_name)
        return ''


@register.tag
def get_object_navigation_links(parser, token):
    args = token.split_contents()
    return GetNavigationLinks(*args[1:])


@register.inclusion_tag('generic_views/generic_navigation.html', takes_context=True)
def object_navigation_template(context):
    return {
        'horizontal': True,
        'object_navigation_links': _get_object_navigation_links(context)
    }
    return new_context



from django.contrib.auth.models import User
from django.conf import settings

AUTO_CREATE_ADMIN = getattr(settings, 'COMMON_AUTO_CREATE_ADMIN', True)
AUTO_ADMIN_USERNAME = getattr(settings, 'COMMON_AUTO_ADMIN_USERNAME', u'admin')
AUTO_ADMIN_PASSWORD = getattr(settings, 'COMMON_AUTO_ADMIN_PASSWORD', User.objects.make_random_password())



import datetime

from django.contrib.auth.models import User, UserManager
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from dynamic_search.api import register
from photos.models import GenericPhoto


class Location(models.Model):
    name = models.CharField(max_length=32, verbose_name=_(u'Name'))
    address_line1 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    address_line2 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    address_line3 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    address_line4 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    phone_number1 = models.CharField(max_length=32, null=True, blank=True, verbose_name=_(u'Phone number'))
    phone_number2 = models.CharField(max_length=32, null=True, blank=True, verbose_name=_(u'Phone number'))

    class Meta:
        ordering = ['name']
        verbose_name = _(u'Location')
        verbose_name_plural = _(u'Locations')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('location_view', [str(self.id)])


class ItemTemplate(models.Model):
    description = models.CharField(verbose_name=_(u'Description'), max_length=64)
    brand = models.CharField(verbose_name=_(u'Brand'), max_length=32, null=True, blank=True)
    model = models.CharField(verbose_name=_(u'Model'), max_length=32, null=True, blank=True)
    part_number = models.CharField(verbose_name=_(u'Part number'), max_length=32, null=True, blank=True)
    notes = models.TextField(verbose_name=_(u'Notes'), null=True, blank=True)
    supplies = models.ManyToManyField('self', null=True, blank=True, verbose_name=_(u'Supplies'))
    suppliers = models.ManyToManyField('Supplier', null=True, blank=True, verbose_name=_(u'Suppliers'))

    class Meta:
        ordering = ['description']
        verbose_name = _(u'Item template')
        verbose_name_plural = _(u'Item templates')

    @models.permalink
    def get_absolute_url(self):
        return ('template_view', [str(self.id)])

    def __unicode__(self):
        return self.description


class Log(models.Model):
    timedate = models.DateTimeField(auto_now_add=True, verbose_name=_(u'Date & time'))
    action = models.CharField(max_length=32, verbose_name=_(u'Action'))
    description = models.TextField(verbose_name=_(u'Description'), null=True, blank=True)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey()

    def __unicode__(self):
        return '%s, %s - %s' % (self.timedate, self.action, self.content_object)

    @models.permalink
    def get_absolute_url(self):
        return ('log_view', [str(self.id)])


class Inventory(models.Model):
    name = models.CharField(max_length=32, verbose_name=_(u'Name'))
    location = models.ForeignKey(Location, verbose_name=_(u'Location'))

    class Meta:
        verbose_name = _(u'Inventory')
        verbose_name_plural = _(u'Inventories')

    @models.permalink
    def get_absolute_url(self):
        return ('inventory_view', [str(self.id)])

    def __unicode__(self):
        return self.name


class InventoryCheckPoint(models.Model):
    inventory = models.ForeignKey(Inventory, verbose_name=_(u'Inventory'))
    datetime = models.DateTimeField(default=datetime.datetime.now(), verbose_name=_(u'Date & time'))
    supplies = models.ManyToManyField(ItemTemplate, null=True, blank=True, through='InventoryCPQty', verbose_name=_(u'Supplies'))


class InventoryCPQty(models.Model):
    supply = models.ForeignKey(ItemTemplate, verbose_name=_(u'Supply'))
    check_point = models.ForeignKey(InventoryCheckPoint, verbose_name=_(u'Check point'))
    quantity = models.IntegerField(verbose_name=_(u'Quantity'))


class InventoryTransaction(models.Model):
    inventory = models.ForeignKey(Inventory, related_name='transactions', verbose_name=_(u'Inventory'))
    supply = models.ForeignKey(ItemTemplate, verbose_name=_(u'Supply'))
    quantity = models.IntegerField(verbose_name=_(u'Quantity'))
    date = models.DateField(default=datetime.date.today(), verbose_name=_(u'Date'))
    notes = models.TextField(null=True, blank=True, verbose_name=_(u'Notes'))

    class Meta:
        verbose_name = _(u'Inventory transaction')
        verbose_name_plural = _(u'Inventory transactions')
        ordering = ['-date', '-id']

    @models.permalink
    def get_absolute_url(self):
        return ('inventory_transaction_view', [str(self.id)])

    def __unicode__(self):
        return "%s: '%s' qty=%s @ %s" % (self.inventory, self.supply, self.quantity, self.date)


class Supplier(models.Model):
    name = models.CharField(max_length=32, verbose_name=_(u'Name'))
    address_line1 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    address_line2 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    address_line3 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    address_line4 = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Address'))
    phone_number1 = models.CharField(max_length=32, null=True, blank=True, verbose_name=_(u'Phone number'))
    phone_number2 = models.CharField(max_length=32, null=True, blank=True, verbose_name=_(u'Phone number'))
    notes = models.TextField(null=True, blank=True, verbose_name=(u'Notes'))

    class Meta:
        ordering = ['name']
        verbose_name = _(u'Supplier')
        verbose_name_plural = _(u'Suppliers')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('supplier_view', [str(self.id)])


register(ItemTemplate, _(u'Templates'), ['description', 'brand', 'model', 'part_number', 'notes'])
register(Location, _(u'Locations'), ['name', 'address_line1', 'address_line2', 'address_line3', 'address_line4', 'phone_number1', 'phone_number2'])
register(Inventory, _(u'Inventory'), ['name', 'location__name'])
register(Supplier, _(u'Supplier'), ['name', 'address_line1', 'address_line2', 'address_line3', 'address_line4', 'phone_number1', 'phone_number2', 'notes'])



from __future__ import absolute_import

from django.conf.urls import patterns, url

from .forms import PreviewForm, ExpressionForm, ImportWizard

urlpatterns = patterns('importer.views',
    url(r'^upload/$', 'import_file', name='import_wizard'),
    url(r'^wizard/$', 'import_wizard', name='import_next_steps'),
    url(r'^download_last_settings/$', 'download_last_settings', name='download_last_settings'),
)





from __future__ import absolute_import

from django.conf.urls import patterns, url
from django.utils.translation import ugettext_lazy as _

from photos.views import generic_photos

from .conf import settings as inventory_settings
from .forms import (LocationForm_view, LogForm, InventoryForm,
                    InventoryTransactionForm, ItemTemplateForm,
                    ItemTemplateForm_view, SupplierForm)
from .models import (Location, Log, Inventory, ItemTemplate,
                     InventoryTransaction, Supplier)
from .views import (InventoryCreateView, InventoryDeleteView,
                    InventoryDetailView, InventoryListView, InventoryUpdateView,
                    LocationCreateView, LocationDeleteView, LocationDetailView,
                    LocationListView, LocationUpdateView, SupplierCreateView,
                    SupplierDeleteView, SupplierDetailView, SupplierListView,
                    SupplierPurchaseOrdersListView, SupplierUpdateView,
                    TemplateCreateView, TemplateDeleteView,
                    TemplateDetailView, TemplateItemsListView, TemplateListView,
                    TemplateOrphanListView, TemplateUpdateView,
                    TransactionDeleteView, TransactionDetailView,
                    TransactionUpdateView)

urlpatterns = patterns('inventory.views',
    url(r'^template/list/$', TemplateListView.as_view(), name='template_list'),
    url(r'^template/create/$', TemplateCreateView.as_view(), name='template_create'),
    url(r'^template/(?P<pk>\d+)/update/$', TemplateUpdateView.as_view(), name='template_update'),
    url(r'^template/(?P<pk>\d+)/delete/$', TemplateDeleteView.as_view(), name='template_delete'),
    url(r'^template/orphans/$', TemplateOrphanListView.as_view(), name='template_orphans_list'),
    url(r'^template/(?P<object_id>\d+)/photos/$', generic_photos, {'model': ItemTemplate, 'max_photos': inventory_settings.MAX_TEMPLATE_PHOTOS, 'extra_context': {'object_name': _(u'item template')}}, 'template_photos'),
    url(r'^template/(?P<pk>\d+)/$', TemplateDetailView.as_view(), name='template_view'),
    url(r'^template/(?P<pk>\d+)/items/$', TemplateItemsListView.as_view(), name='template_items_list'),
    url(r'^template/(?P<object_id>\d+)/assign/supplies$', 'template_assign_remove_supply', (), name='template_assign_supply'),
    url(r'^template/(?P<object_id>\d+)/assign/suppliers/$', 'template_assign_remove_suppliers', (), name='template_assign_suppliers'),

    url(r'^inventory/list/$', InventoryListView.as_view(), name='inventory_list'),
    url(r'^inventory/create/$', InventoryCreateView.as_view(), name='inventory_create'),
    url(r'^inventory/(?P<pk>\d+)/$', InventoryDetailView.as_view(), name='inventory_view'),
    url(r'^inventory/(?P<pk>\d+)/update/$', InventoryUpdateView.as_view(), name='inventory_update'),
    url(r'^inventory/(?P<pk>\d+)/delete/$', InventoryDeleteView.as_view(), name='inventory_delete'),
    url(r'^inventory/(?P<object_id>\d+)/transaction/create/$', 'inventory_create_transaction', (), 'inventory_create_transaction'),
    url(r'^inventory/(?P<object_id>\d+)/transaction/list/$', 'inventory_list_transactions', (), 'inventory_list_transactions'),

    url(r'^transaction/(?P<pk>\d+)/$', TransactionDetailView.as_view(), name='inventory_transaction_view'),
    url(r'^transaction/(?P<pk>\d+)/update/$', TransactionUpdateView.as_view(), name='inventory_transaction_update'),
    url(r'^transaction/(?P<pk>\d+)/delete/$', TransactionDeleteView.as_view(), name='inventory_transaction_delete'),

    url(r'^location/list/$', LocationListView.as_view(), name='location_list'),
    url(r'^location/create/$', LocationCreateView.as_view(), name='location_create'),
    url(r'^location/(?P<pk>\d+)/update/$', LocationUpdateView.as_view(), name='location_update'),
    url(r'^location/(?P<pk>\d+)/delete/$', LocationDeleteView.as_view(), name='location_delete'),
    url(r'^location/(?P<pk>\d+)/$', LocationDetailView.as_view(), name='location_view'),

    url(r'^supplier/(?P<pk>\d+)/$', SupplierDetailView.as_view(), name='supplier_view'),
    url(r'^supplier/list/$', SupplierListView.as_view(), name='supplier_list'),
    url(r'^supplier/create/$', SupplierCreateView.as_view(), name='supplier_create'),
    url(r'^supplier/(?P<pk>\d+)/update/$', SupplierUpdateView.as_view(), name='supplier_update'),
    url(r'^supplier/(?P<pk>\d+)/delete/$', SupplierDeleteView.as_view(), name='supplier_delete'),
    url(r'^supplier/(?P<object_id>\d+)/assign/itemtemplates/$', 'supplier_assign_remove_itemtemplates', (), 'supplier_assign_itemtemplates'),
    url(r'^supplier/(?P<pk>\d+)/purchase/orders/$', SupplierPurchaseOrdersListView.as_view(), name='supplier_purchase_orders'),
)



from __future__ import absolute_import

import os

from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse, reverse_lazy
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _

from common.utils import encapsulate

from .conf import settings as photos_settings
from .forms import PhotoForm
from .models import GenericPhoto


def generic_photos(request, model, object_id, max_photos=5, extra_context={}):
    model_instance = get_object_or_404(model, pk=object_id)
    photos = GenericPhoto.objects.photos_for_object(model_instance)

    if request.method == 'POST' and photos.count() < max_photos:
        form = PhotoForm(request.POST, request.FILES)
        if form.is_valid():
            instance = form.save(commit=False)
            if instance.image.size > photos_settings.MAX_PHOTO_SIZE:
                messages.error(request, _(u'The photo is too big.'))
                os.unlink(instance.photo.path)
                return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse('home')))

#               ext = os.path.splitext(os.path.split(instance.get_photo_filename()))
#               if ext != '.png' and ext != '.jpg' and ext != '.gif':
#                   _flash_message(request, _(u'El solo fotos de tipo: PNG, JPG o GIF son permitidas.'), type='error')
#                   os.unlink(instance.get_photo_filename())
#                   return HttpResponseRedirect('/photos/' + str(ad.number))

            instance.object_id = object_id
            instance.content_type = ContentType.objects.get_for_model(model)
            instance.save()

            photos = GenericPhoto.objects.photos_for_object(model_instance)
            if photos.filter(main=True).count() == 0:
                new_main_photo = photos[0]
                new_main_photo.main = True
                new_main_photo.save()

            messages.success(request, _(u'The photo was added successfully.'))
            return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse('home')))
    else:
        form = PhotoForm()

    extra_context.update({
        'title': _(u'photos for%(object_name)s%(max_photos)s: %(object)s') % {
                'object_name': ' %s' % unicode(extra_context['object_name']) if 'object_name' in extra_context else '',
                'object': model_instance,
                'max_photos': (_(u' (maximum of %s)') % max_photos if max_photos else '')
            },
        'object': model_instance,
        'object_list': photos,
        'hide_object': True,
        'extra_columns': [
            {'name': _(u'Photo'), 'attribute': encapsulate(lambda x: '<div class="gallery"><a href="%s"><img src="%s" /></a></div>' % (x.get_display_url(), x.get_thumbnail_url()))},
            {'name': _(u'Main photo'), 'attribute': encapsulate(lambda x: '<span class="famfam active famfam-accept"></span>' if x.main else '-')},
            {'name': _(u'Title'), 'attribute': encapsulate(lambda x: x.title if x.title else '-')}
        ],
    })

    if GenericPhoto.objects.photos_for_object(model_instance).count() < max_photos:
        extra_context.update({
            'subforms_dict': [
                {
                    'name': 'generic_views/generic_form_subtemplate.html',
                    'title': _(u'Upload new photo'),
                    'form': form,
                },
            ],
        })
    else:
        # subform_dict is persistent between views, clear it explicitly
        extra_context.update({'subforms_dict': []})

    return render_to_response('photos/photos.html', extra_context,
        context_instance=RequestContext(request))


def generic_photo_mark_main(request, object_id):
    photo = get_object_or_404(GenericPhoto, pk=object_id)

    if request.method == 'POST':
        photos = GenericPhoto.objects.photos_for_object(photo.content_object)
        photos.update(main=False)
        photo.main = True
        photo.save()

        messages.success(request, _(u'The main photo has been changed.'))
        return HttpResponseRedirect(request.POST.get('next', '/'))

    return render_to_response('generic_views/generic_confirm.html', {
        'next': request.META.get('HTTP_REFERER', reverse('home')),
        'previous': request.META.get('HTTP_REFERER', reverse('home')),
        'object': photo.content_object,
    }, context_instance=RequestContext(request))


def generic_photo_delete(request, object_id):
    photo = get_object_or_404(GenericPhoto, pk=object_id)
    if request.method == 'POST':
        main = photo.main
        photo.delete()
        if main:
            photos = GenericPhoto.objects.photos_for_object(photo.content_object)
            if photos.count() > 0:
                new_main_photo = photos[0]
                new_main_photo.main = True
                new_main_photo.save()

                messages.success(request, _(u'The main photo was deleted successfully.  The another photo has been selected as the main photo.'))
            else:
                messages.success(request, _(u'The main photo was deleted successfully.'))
        else:
            messages.success(request, _(u'The photo was deleted successfully.'))

        return HttpResponseRedirect(request.POST.get('next', reverse('home')))

    return render_to_response('generic_views/generic_confirm.html', {
        'next': request.META.get('HTTP_REFERER', reverse('home')),
        'previous': request.META.get('HTTP_REFERER', reverse('home')),
        'title': _(u'Are you sure you wish to delete this photo?'),
        'object': photo.content_object,
        'delete_view': True,
    }, context_instance=RequestContext(request))
